function [y,i] = sort(x,dim,mode)
% SORT   Sort in ascending or descending order.  The sorting is
%     done by degree and then lexicographic ordering.
%
%     For vectors, SORT(X) sorts the elements of X in ascending order.
%     For matrices, SORT(X) sorts each column of X in ascending order.
%  
%     Y = SORT(X,DIM,MODE)
%     has two optional parameters.  
%     DIM selects a dimension along which to sort.
%     MODE selects the direction of the sort
%        'ascend' results in ascending order
%        'descend' results in descending order
%     The result is in Y which has the same shape and type as X.
%  
%     [Y,I] = SORT(X,DIM,MODE) also returns an index matrix I.
%     If X is a vector, then Y = X(I).  
%     If X is an m-by-n matrix and DIM=1, then
%         for j = 1:n, Y(:,j) = X(I(:,j),j); end
%  

% 4/27/2006: PJS  Initial Coding  

% Error Checking
if nargin==1
    dim = 1;
    mode = 'ascend';
elseif nargin ==2
    mode = 'ascend';
elseif nargin~=3
    error('[y,i] = sort(x,dim,mode)');
end

if isempty(dim) || ~isa(dim,'double') || any( size(dim)~=[1 1]) ...
        || floor(dim)~=ceil(dim)
    error('DIM must be a positive integer.')
elseif dim~=1 & dim~=2
    error('DIM must be within array bounds.');
end
if ~strcmp(mode,'ascend') & ~strcmp(mode,'descend');
    error('sorting direction must be ''ascend'' or ''descend''.');
end

% Sort along second dim:
% Transpose, do sort on columns, and then transpose back
szx0 = size(x);
if dim==2 || (nargin==1 & szx0(1)==1)
    x = x.';
    szx = size(x);
else
    szx = szx0;
end

% Do Sort Column-by-Column
flipidx = szx(1):-1:1;
y = polynomial(zeros(szx));
i = zeros(szx);
L.type = '()';
for i1 = 1:szx(2)
    L.subs = {':',i1};
    xcol = subsref(x,L);    
    [xcol,icol] = LOCALQuickSort(xcol,1:szx(1));
    if strcmp(mode,'ascend')        
        L.subs = {flipidx,1};
        xcol = subsref(xcol,L);
        icol = icol(flipidx);
    end
    L.subs = {':',i1};
    y = subsasgn(y,L,xcol);    
    i(:,i1) = icol;
end

if dim==2 || (nargin==1 & szx0(1)==1)
    y = y.';
    i = i.';
end
    
    
%-------------- Quicksort on a column of polynomials----------------    
function [y,i]=LOCALQuickSort(x,xidx);

xlen = length(x);
if xlen<=1
    y = x;
    if isempty(x)
        i=[];
    else
        i = xidx;
    end
    return
else
    % Do QuickSort
    pidx = floor(xlen/2);
    L.type = '()';
    L.subs = {pidx};
    pivot = subsref(x,L);
    U = []; Uidx = [];
    G = []; Gidx = [];
    for i1 = [1:(pidx-1), (pidx+1):xlen]
        L.subs = {i1};
        xi = subsref(x,L);
        if xi >= pivot
            U = [U; xi];
            Uidx = [Uidx; xidx(i1)];
        else
            G = [G; xi];
            Gidx = [Gidx; xidx(i1)];
        end
    end
    [Us,Usidx] = LOCALQuickSort(U,Uidx);
    lU = length(Uidx);
    [Gs,Gsidx] = LOCALQuickSort(G,Gidx);
    y = [Us; pivot; Gs];
    i = [Usidx; xidx(pidx); Gsidx];
end
 
% a=[z^6; x1*x2*z+3; b*z; x1*x2; 6; 8*x1];
% [c,i]=sort(a,1,'descend')
