%% Van Der Pol Region of Attraction Problem
% This demo will demonstrate how to use the ROA analysis tools.  By
% increasing the degree of the Lyapunov function V(x) used to estimate the
% region of attraction, a larger ROA will be found.

%%
format('compact')
clear all
close all

%% Problem Statement:

%%
% Given f, L1, L2, p, this code computes solutions to the problem
% maximize  beta by choice of V, beta
% subject to:
% 
% $$V(0) = 0$$
% 
% $$V - L1 >= 0$$
%
% $$\{x : p(x) \le beta\} \subset \{x : V(x) \le 1\} \subset \{x : Vdot(x) < 0\}$$
%
% where Vdot = jacobian(V,x)*f(x)

%%
% This code solves SOS relaxations for the above problem following the
% procedure:
%%
% 1. Generate an initial V feasible for the above constraints.
%        - use simulation data + linearization
%        - use only linearization
%%
% 2. Improve the estimate of the ROA by further optimization, namely
% iterating between optimizing over the choice of "multipliers" for given V
% and optimizing over the choice of V given the multipliers. 

%% Setup Dynamics
% Form the vector field
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
f= [x1dot; x2dot];

%%
% Plot Van Der Pol limit cycle
plotVDP
domain = [-3 3 -3 3];
axis(domain)

%% Quadratic V(x)
% *Extract default options.*  The software uses two different iterations:
% one requires bisection and one does not.  The default does not require
% bisection.  The SOS multipliers in the two different iterations are
% defined as follows:
%
% 1. Without bisection: r1, r2 (these are polynomials in x - non necessarily
% SOS)
%
% 2. With bisection: s1, s2, s3 (all SOS)
%
% Most of the slides are written based on the conditions that require
% bisection. Therefore, to ensure that we are using bisection we set
% Bis.flag=1 (default = 0). 
Bis.flag = 1;

%%
% *Generate the default options used for the rest of calculations.*  We
% omit the 3rd and 4th input arguments for now.

[roaconstr,opt,sys] = GetRoaOpts(f, x, [],[],Bis);
%%
% *Set custom options.*
% We adjust some of the tolerance levels in order to get better results.
% By decreasing the iterations and increasing the tolerances, we get
% worse results, but the program runs faster.
opt.sim.NumConvTraj = 100;
opt.coordoptim.MaxIters = 30;
opt.coordoptim.IterStopTol = 1e-4;
opt.getbeta.bistol = 1e-4;
opt.getgamma.bistol = 1e-4;
opt.display.roaest = 1;

%%
% If Bis.flag = 1, then the default basis vectors for s1, s2, s3 are the
% following. (si is constructed as si = zi*Mi*zi with Mi SOS)
z1 = roaconstr.z1
z2 = roaconstr.z2
z3 = roaconstr.z3

%%
%  V is constructed as follows: V(x)= A*zV, where A is a row vector of
%  decision variables. The default vector field for V is quadratic.
zV = roaconstr.zV

%%
% We will use the unit ball as the default shape for p. p(x)=x'*x
roaconstr.p

%%
% We will use 1e-6*x'*x as the default shape for L1 and L2.  L1 and L2 are
% small positive definite sums of squares. 
roaconstr.L1
roaconstr.L2

%%
% *Estimate the ROA with Quadratic V(x).* The function wrapper estimates
% the ROA.  The second input argument is empty because our vector field, f,
% does not have any uncertainty.
outputs = wrapper(sys,[],roaconstr,opt);
%%
% Get the V, betas and multipliers
[V,betaLower,gamma,p,multip,betaUpper] = extractSol(outputs);

V
s1 = multip.S1
s2 = multip.S2
betaLower
betaUpper

%%
% Verify polynomials are SOS
issos(V)
issos(s1)
issos(s2)

%%
% *Display Results.* Plot p(x) and Quadratic V(x)
hold on;
betaLower=double(betaLower);
pcontour(V,gamma,domain,'r')
pcontour(p,betaLower, domain,'g')
title('ROA Estimation of Van Der Pol with Quadratic V(x)')
legend('Van Der Pol', 'V(x)', 'p(x)', 'Location', 'NorthWest')
axis(domain)

%% Quartic V(x)
% Get default options again. This time zV specifies that V is quartic.
zV = monomials(x, 2:4);
[roaconstr,opt,sys] = GetRoaOpts(f, x, zV,[],Bis);

%%
% A larger ROA is obtained by searching over quartic s2 polynomials
roaconstr.z2 = monomials(x, 1:2);

%%
% *Set custom options.* We need to specify the settings again.
opt.sim.NumConvTraj = 100;
opt.coordoptim.MaxIters = 30;
opt.coordoptim.IterStopTol = 1e-4;
opt.getbeta.bistol = 1e-4;
opt.getgamma.bistol = 1e-4;
opt.display.roaest = 0;

%%
% Estimate the ROA with Quartic V(x)
outputs = wrapper(sys,[],roaconstr,opt);
%%
% Get the V, betas and multipliers
[V,betaLower,gamma,p,multip,betaUpper] = extractSol(outputs);

V
s1 = multip.S1
s2 = multip.S2
betaLower
betaUpper

%%
% Verify polynomials are SOS
issos(V)
issos(s1)
issos(s2)

%%
% Plot p(x) and V(x)
figure;
hold on;
plotVDP
pcontour(V,gamma,domain,'r')
pcontour(p,betaLower, domain,'g')
title('ROA Estimation of Van Der Pol with Quartic V(x)')
legend('Van Der Pol', 'V(x)', 'p(x)', 'Location', 'NorthWest')
axis(domain)

%% Degree 6 V(x)
% Get default options again. This time zV specifies that the degree of V is
% 6.
zV = monomials(x, 2:6);
[roaconstr,opt,sys] = GetRoaOpts(f, x, zV,[],Bis);

%%
% A larger ROA is obtained by searching over quartic s2 polynomials
roaconstr.z2 = monomials(x, 1:2);

%%
% *Set custom options.* We need to specify the setting again.
opt.sim.NumConvTraj = 100;
opt.coordoptim.MaxIters = 30;
opt.coordoptim.IterStopTol = 1e-4;
opt.getbeta.bistol = 1e-4;
opt.getgamma.bistol = 1e-4;
opt.display.roaest = 0;

%%
% Estimate the ROA with degree 6 V(x)
% wrapper computes the ROA estimation routine.  The second input argument
% is empty because our vector field, f, does not have any
% uncertainty.  See "help wrapper" for instructions on specifying uncertain
% vector fields.
outputs = wrapper(sys,[],roaconstr,opt);
%%
% Get the V, betas and multipliers
[V,betaLower,gamma,p,multip,betaUpper] = extractSol(outputs);

V
s1 = multip.S1
s2 = multip.S2
betaLower
betaUpper

%%
% Verify polynomials are SOS
issos(V)
issos(s1)
issos(s2)

 
%%
% Plot p(x) and degree 6 V(x) 
figure;
hold on;
plotVDP
pcontour(V,gamma,domain,'r')
pcontour(p,betaLower, domain,'g')
title('ROA Estimation of Van Der Pol with degree of V(x) = 6')
legend('Van Der Pol', 'V(x)', 'p(x)', 'Location', 'NorthWest')
axis(domain)


%%
% For details of iterations with no bisection see:
% 
% _Topcu, Seiler, and Packard, "Local Stability Analysis Using Simulations and 
% Sum-of-Squares Programming," Automatica, 2008 or the slide titled
% "Application of Set Containment Conditions (2)."_

