function [hk_sol, RRefine]= reachRefine(f,x,w,V,R,NumberAnnuli,options)
% function [hk_sol, RRefine]=
% reachRefine(f,x,w,Vvec,R,NumberAnnuli, options)
%
% DESCRIPTION
%  Subdivide {x: V(x)<=R} into smaller annular regions to refine the
%  original upper bound R.
%
%  Solves the following problem:
%  minimize hk by choice of s1k >=0, s2k >=0
%  Subject to:
%  -((epsilon*k -V)*s1k + (V - epsilon*(k-1))*s2k + Vdot -hk*w'*w) >= 0
%  for k=1...NumberAnnuli, with epsilon*NumberAnnuli=R^2, 
%
%  The refined R values are calculated as follows:
%  RRefine=sqrt(sum(ones(1,NumberAnnuli)./hk_sol)*epsilon);
%
% INPUTS
%   f: nonlinear system with pvars as states
%   x: state vector of pvars  
%   w: input vector of pvars
%   V: polynomial V from REACH
%   R: scalar value of R from original REACH calculation
%   NumberAnnuli: Number of Refinement iterations (i.e. divisions of
%   annuli)
%   options: You have the option to input your own basis vector for the s
%   polynomials.  Set options.sBasis = monomials(p, deg), where p is a
%   vector of pvars and deg is a vector of non-negative integers specifying
%   the degree of monomials to be included in the sBasis.  When choosing
%   deg, take into account that the s polynomials are chosen to be in
%   matrix form.  For example, you should set options.sBasis =
%   monomials([x;w],1:s_degree/2), where s_degree is the desired degree of
%   s .  If this field empty, an appropriate basis vector is
%   calculated.  
%
% OUTPUTS
%   hk_sol: vector of hk's found from each refinement iteration point
%   RRefine: vector of refined R values , given by the equation
      


pvar hk

if nargin >= 7   
    zS=options.sBasis;    
else
    if mod(f.maxdeg,2) == 1 
        s_degree = f.maxdeg - 1;
    else
        s_degree = f.maxdeg;
    end
    zS = monomials([x;w],1:s_degree/2);
end


s1 = polydecvar('c1',zS, 'mat');
s2 = polydecvar('c2',zS, 'mat');

sosconstr_first = cell(2,1);
sosconstr = cell(3,1);

epsilon = R^2/NumberAnnuli;   
Vdot = jacobian(V,x)*f;
opts.fid = 0;
opts.checkfeas  = 'full';
for k=1:NumberAnnuli

    if k ~=1
        %Constraint 1: s1k >= 0
        sosconstr{1} = s1;

        %Constraint 2: s2k >= 0
        sosconstr{2} = s2;

        %Constraint 3: -((epsilon*k - V)*s1k + (V - epsilon*(k-1))*s2k + Vdot -hk*w'w ) >=0
        sosconstr{3} = -((epsilon*k -V)*s1 + (V - epsilon*(k-1))*s2 + Vdot -hk*w'*w);

        % Set Objective: minimize hk
        obj = hk ; 

        [info,dopt,sossol] = sosopt(sosconstr,[x;w],obj,opts);

    else
        %Constraint 1: s1k >= 0
        sosconstr_first{1} = s1;

        %Constraint 2: -((epsilon*k - V)*s1k + Vdot -hk*w'w ) >=0        
        sosconstr_first{2} = -((epsilon*k -V)*s1 + Vdot -hk*w'*w);

        % Set Objective: minimize hk
        obj = hk ; 
        
        [info,dopt,sossol] = sosopt(sosconstr_first,[x;w],obj,opts);


    end
     hk_sol(k)= subs(hk,dopt(:,1),dopt(:,2));
     s1_sol = subs(s1,dopt(:,1),dopt(:,2));
     s2_sol = subs(s2,dopt(:,1),dopt(:,2));

end
hk_sol = double(hk_sol);

RRefine=sqrt(sum(ones(1,NumberAnnuli)./hk_sol)*epsilon);    