function yd = f18full(t,x,u)
%
% yd = f18full(t,x,u)
% 
% This function contains a description of the six DoF F/A-18 model with
% a complete description of the aerodynamic model.  The references have
% been cited whenever any data have been presented.  This also contains the
% physical parameters for the aircraft. 
%
% x represents the states. The state ordering is: 
% 
% V       =  x(1);       % Airspeed , ft/s
% beta    =  x(2);       % Sideslip Angle, rad
% alpha   =  x(3);       % Angle-of-attack, rad
% 
% p       =  x(4);       % Roll rate, rad/s
% q       =  x(5);       % Pitch rate, rad/s
% r       =  x(6);       % Yaw rate, rad/s
% 
% phi     =  x(7);       % Roll Angle, rad
% theta   =  x(8);       % Pitch Angle, rad
% psi     =  x(9);       % Yaw Angle, rad
%
% u represents the inputs. The input ordering is:
%      
% d_STAB   = u(1);       % Stabilator Deflection, rad
% d_RUD    = u(2);       % Rudder Deflection, rad
% d_AIL    = u(3);       % Aileron Deflection, rad
%
% NOTE:
% We have not used this model for any of the analysis results presented 
% later. However, this model has been provided for completeness. We will use 
% a reduced order model for our linear and nonlinear analysis purpose. We
% have not used this model for any of the analysis results presented later.
% 


%==========================================================================
% Aircraft Physical Paramters
% Reference: S. B. Buttrill and P. D. Arbuckle  and K. D. Hoffler
%            Simulation model of a twin-tail, high performance airplane
%            NASA 1992, NASA TM-107601


S = 400;                % ft^2
b =  37.4;                % ft
c =  11.52;             % ft
rho = 1.0660e-003;      % slugs/ft^3  --- 25C / 25000 ft
Ixx = 23000;            % slugs*ft^2
Iyy = 151293;           % slugs*ft^2 
Izz = 169945;           % slugs*ft^2 
Ixz = - 2971;           % slugs*ft^2
m = 1034.5;             % slugs
g = 32.2;               % ft/s^2

% Unit Conversion : Degree <--> Radian
d2r = pi/180;
r2d = 1/d2r;



%==========================================================================
% State ordering 


V       =  x(1);       % Airspeed , ft/s
beta    =  x(2);       % Sideslip Angle, rad
alpha   =  x(3);       % Angle-of-attack, rad

p       =  x(4);       % Roll rate, rad/s
q       =  x(5);       % Pitch rate, rad/s
r       =  x(6);       % Yaw rate, rad/s

phi     =  x(7);       % Roll Angle, rad
theta   =  x(8);       % Pitch Angle, rad
psi     =  x(9);       % Yaw Angle, rad


% Input Terms
       
d_STAB   = u(1);       % Stabilator Deflection, rad
d_RUD    = u(2);       % Rudder Deflection, rad
d_AIL    = u(3);       % Aileron Deflection, rad
        
T        = 1000;       % Thrust, assumed constant
        
%==========================================================================
% Aerodynamic Model 
% 
% Aerodynamic Data have been extracted from the following references. 
%
% References: 
%
% (1) Kenneth W. Iliff  and Kon-Sheng C. Wang; 
%     Extraction of lateral-directional stability and  control derivatives 
%     for the basic F-18 aircraft at high angles of attack. 
%     NASA TM-4786 , 1997.
% 
% (2) Marcello R. Napolitano and Alfonso C. Paris and Brad A. Seanor,
%     Estimation of the lateral-directional aerodynamic parameters from 
%     flight data for the NASA F/A-18 HARV.
%     AIAA Atmospheric Flight Mechanics Conference,
%     AIAA-96-3420-CP, 1996, 479-489. 
%
% (3) Marcello R. Napolitano and Alfonso C. Paris and Brad A. Seanor,
%     Estimation of the longitudinal aerodynamic parameters from flight 
%     data for the NASA F/A-18 HARV
%     AIAA Atmospheric Flight Mechanics Conference,
%     AIAA-96-3420-CP, 1996, 469-478.
%
% (4) Marcello R. Napolitano and Joelle M. Spagnuolo,
%     Determination of the stability and control derivatives of the 
%     NASA F/A-18 HARV using flight data, NASA CR-194838, 1993.
%
% (5) Lulch C. Daniel, Analysis of the Out-of-Control Falling Leaf Motion
%     using a Rotational Axis Coordinate System. MS thesis
%     Virginia Polytechnic Institue and State Unniversity, 1998.


% Rolling Moment   
C_l = (-2.0175*alpha^3 +  3.4510*alpha^2 - 1.6156*alpha +0.045791)*beta ...
        + (-1.5526e-001*alpha + 1.5417e-001)* d_AIL ...
        + (-1.8583e-002*alpha +  1.9433e-002)*d_RUD ...
        + (2.0054e-001*alpha -3.3700e-001)*p*b/2/V;


%  Yawing Moment
C_n =   (-3.9165e-001*alpha^2 +  3.6480e-002*alpha +  8.9354e-002)*beta...
       +  (-0.0213*alpha +  0.0051)*d_AIL ...
       +  (5.3361e-002*alpha -7.2422e-002)*d_RUD ...
       +  (-7.1620e-002 *alpha-4.3750e-001)*r*b/2/V;
   
% SideForce
C_Y   = ( -6.7491e+000*alpha^3 +  9.4127e+000 *alpha^2 -2.9649*alpha ...
         - 5.6545e-001)*beta + (-4.4857e-001*alpha +  3.0794e-001)*d_RUD ...
         +  ( 4.2702e-001*alpha -1.0472e-001)*d_AIL;
         
% Pitching Moment           
C_m =  (-0.9931*alpha + 0.1407) +  (0.6401*alpha -1.1055) * d_STAB ...
       +  ( -14.3*alpha - 2)*c*q/2/V;

% Normal Force   
C_normal = (-0.24621*alpha + 8.8808e-001)*d_STAB ...
           + ( 42.97*alpha + 6 )*c*q/2/V;
                        
% Axial Force 
C_axial =  (-0.0092*alpha + 0.0085)*d_STAB ...
            + (-0.0208e-003 *alpha +  9.2366 )*c*q/2/V ;


% Lift Coefficient
C_lift = (-0.0204 + 5.677*alpha - 5.4246*alpha^2 + 1.1645*alpha^3)*cos(2*beta/3)...
          +  C_normal*cos(alpha) - C_axial*sin(alpha);

      
% Drag Coefficient
C_drag =  (-1.4994 - 0.1995*alpha + 6.3971*alpha^2 - 5.7341*alpha^3 + ....
           1.4610*alpha^4) *cos(beta) + 1.5036...
          + ( C_normal*sin(alpha) + C_axial*cos(alpha));
 
     
      
% Form Aerodynamic forces and moments

qbar = 1/2*rho*V^2;  % Dynamic pressure
L =  C_l*qbar*S*b ;
M =  C_m*qbar*S*c;
N =  C_n*qbar*S*b;
Y =  C_Y*qbar*S ;
Lift = C_lift*qbar*S ;
Drag = C_drag*qbar*S ;

% Body to Wind Axis Conversion of the Aerdynamic data
CD_w = C_drag*cos(beta) - C_Y*sin(beta);
CY_w = C_Y*cos(beta) + C_drag*sin(beta);

%==========================================================================
% Equations of Motion
%
% References: 
% 
% (i) Determination of the stability and control derivatives of the 
%     NASA F/A-18 HARV using flight data; 
%     Marcello R. Napolitano and Joelle M. Spagnuolo,NASA CR-194838, 1993. 


%--------------------------------------------------------------------------
% Force Equation 

Vd =  -qbar*S*CD_w /m + g*(cos(phi)*cos(theta)*sin(alpha)*cos(beta) + ...
     sin(phi)*cos(theta)*sin(beta) -sin(theta)*cos(alpha)*cos(beta)) ...
     + (T/m)*cos(beta)*cos(alpha);

betad =  qbar*S*CY_w/m/V + p*sin(alpha)-r*cos(alpha) + ....
         g*cos(theta)*sin(phi)*cos(beta)/V + sin(beta)*(g*cos(alpha)*sin(theta)...
         - g*sin(alpha)*cos(phi)*cos(theta)+ T*cos(alpha)/m)/V;

alphad = -Lift/m/V/cos(beta) + q - tan(beta)*(p*cos(alpha)+r*sin(alpha))...
          + g*(cos(phi)*cos(theta)*cos(alpha) + sin(theta)*sin(alpha))/V/cos(beta)...
          -(T/m/V/cos(beta))*sin(alpha);
      

%--------------------------------------------------------------------------
%Moment Equations

pd = ((Izz* L + Ixz* N - (Ixz*(Iyy-Ixx-Izz)*p + ...
     (Ixz^2 + Izz*(Izz - Iyy))*r)*q)/(Ixx*Izz -Ixz^2));
 
qd = ((M + (Izz -Ixx)*p*r + (r^2 -p^2)*Ixz)/Iyy);

rd =  ((Ixz * L  + Ixx * N + (Ixz * (Iyy - Ixx -Izz) *r +...
      (Ixz^2 + Ixx*(Ixx - Iyy))*p)*q)/(Ixx*Izz -Ixz^2));



%--------------------------------------------------------------------------
% Kinetics Equation

phid = p + (q*sin(phi) + r*cos(phi))*tan(theta);

thetad = q*cos(phi) - r*sin(phi);

psid =  (q*sin(phi) + r*cos(phi))*sec(theta);



%--------------------------------------------------------------------------
% Group state derviatives

yd = [Vd ;betad; alphad; pd; qd; rd; phid; thetad; psid]; 

