%
% Objectives: 
% This file trims and linearizes the aircraft at the given flight
% condition.  
% 

%========================================================================== 
% Load F/A-18 data
% 
% This data file initializes F/A-18 Physical paramters data and the
% Velocity ( since it is assumed constant). 

f18_data; 

%==========================================================================
% Statename , Inputname and Outputname
%
% Baseline and Revised has different feedback channels 
% So, their output equations are different

statenames = {'Beta (rad)','Roll Rate (rad/s)','Yaw Rate (rad/s)',...
            'Phi (rad)','Alpha (rad)','Pitch Rate (rad/s)'}';
inputnames = { 'Aileron (rad)','Rudder (rad)','Stabilator (rad)'}';       


%==========================================================================
% Baseline Output Equation 
%==========================================================================

% The baseline control law uses lateral acceleration (Ay) as one of the 
% feedback channel which is not included in the C matrix of Pss; hence, 
% we will have to augment the Ay in the output equation (Pss.C) 

% qbar =  1/2*rho*V^2;  % Dynamic pressure

%--------------------------------------------------------------------------
% Sideforce Coefficient for Ay feedback

% C_Y        = ( 0.5781*alpha^2 +0.2834*alpha -0.8615)*beta...
%              + (-4.4857e-001*alpha +  3.0794e-001)*d_RUD ...
%              +  ( 4.2702e-001*alpha -1.0472e-001)*d_AIL;
%          
%          
% x = [beta;p1;r;phi;alpha;q];  % State Vector
% u = [d_AIL; d_RUD; d_STAB];   % Input Vector

%--------------------------------------------------------------------------
% Output Equation

% Ay = qbar*S*(C_Y)/m/g;
% y = [Ay; p1; r; alpha;q];
% J = jacobian(y,u);
% H = y-J*u;


outputnamesBaseline = {'Lat Acc (g)','Roll Rate (rad/s)','Yaw Rate (rad/s)',...
            'Alpha (rad)','Pitch Rate (rad/s)'}';

 Baseline_C = [-0.3446     0         0         0         0         0;...
                 0         1         0         0         0         0;...
                 0         0         1         0         0         0;...
                 0         0         0         0         1         0;...
                 0         0         0         0         0         1]; 
             
Baseline_D = [   -0.0419    0.1232     0;...
                   0         0         0;...
                   0         0         0;...
                   0         0         0;...
                   0         0         0 ];        

%==========================================================================
% Revised Output Equation 
%==========================================================================

% The revised control law uses Ay, beta, and betadot as feedback 
% channels which are not included in the C matrix of Pss;; hence, 
% we will have to augment the output equation (Pss.C) with these terms. 
% For betadot feedback, we will use the linearized kinematic equation of 
% betadot as feedback meaurement. 


%-------------------------------------------------------------------------- 
% Revised Control Law

% Y    =  C_Y*qbar*S ;
% Czx = (0.53594*alpha -3.8841*alpha^2 -1.6387*beta^2) + ...
%       ( 0.027584 - 0.77719 *alpha)*d_STAB;
% betad1 =   ( Y  - qbar*S*Czx*beta)/m/V + g*(phi)/V + p1*alpha - r ;


%--------------------------------------------------------------------------
% Output Equation

% Ay = qbar*S*(C_Y)/m/g;
% y = [Ay; p1; r; alpha;beta;q;betad1];
% J = jacobian(y,u);
% H = y-J*u;


outputnamesRevised = {'Lat Acc (g)','Roll Rate (rad/s)','Yaw Rate (rad/s)',...
            'Alpha (rad)','Beta (rad)','Pitch Rate (rad/s)','Betadot'}';
        
Revised_C = [-0.3446         0         0         0         0         0;...
                0       1.0000         0         0         0         0;...
                0         0       1.0000         0         0         0;...
                0         0            0         0       1.0000      0;...
            1.0000        0            0         0         0         0;...
                0         0            0         0         0     1.0000;...
           -0.0444        0       -1.0000    0.1288        0         0];
       
Revised_D  = [-0.0419    0.1232      0;...
                 0         0         0;...
                 0         0         0;...
                 0         0         0;...
                 0         0         0;...
                 0         0         0;...
             -0.0054    0.0159       0];
         
%========================================================================== 
% Trimming initial conditions
%========================================================================== 

% This Sections Generates linear plants at the flight conditions specified
% below. 

alpha   = 10*d2r; 
beta    = 0*d2r; 
phi     = 0*d2r;  
p       = 0*d2r; 
q       = 0*d2r; 
r       = 0*d2r; 

% Initialize Input Value

d_STAB   = 0*d2r;
d_AIL    = 0*d2r;
d_RUD    = 0*d2r;


u_init = [d_AIL; d_RUD; d_STAB] ;
x_init = [beta;p;r;phi;alpha;q] ;



%==========================================================================
% Operating Point Specificaton Setup

opys = operspec('f18_trim');

opys.States(1).Known       = [    1;     0;    0;    1;      1 ;   0 ];

opys.States(1).x           = [   beta;   p;    r;   phi;   alpha;  q ];

opys.States(1).steadystate = [   1;      1;    1;    1;      1;    1];


%Setting the Input value

opys.inputs(1).known = 0;
opys.inputs(2).known = 0;
opys.inputs(3).known = 0;


opys.inputs(3).u = d_STAB;
opys.inputs(2).u = d_RUD;
opys.inputs(1).u = d_AIL;


%==========================================================================
% Finding Trim/ Operating point
opt1 = optimset('MaxFunEvals',1e+04);
opt = linoptions('OptimizationOptions',opt1);
[ysop,rep] = findop('f18_trim',opys,opt);
get(ysop);



%==========================================================================
% Extracting Trim Point

x_trim = [ysop.States.x(1);  ysop.States.x(2); ysop.States.x(3);...
          ysop.States.x(4);  ysop.States.x(5); ysop.States.x(6)]; 


u_trim = [ysop.Inputs(1).u; ysop.Inputs(2).u; ysop.Inputs(3).u];      




%==========================================================================
% Creating Open Loop Linearized Model 




[A ,B ,C, D] = linmod('f18_trim',x_trim, u_trim);

linf18 = ss(A,B,C,D,'statename',statenames, ...
   'inputname',inputnames,...
   'outputname',{'Beta','Roll','Yaw','Phi','Alpha','Pitch'});


% Open Loop Model
Pss = linf18;

