%
% This file creates an open loop model for the reduced F/A-18 Model. 
% The reduced model represents rolling maneuver for aircraft. 
% 
% Polynomial toolbox is needed to run this file. Since the State and the
% Input variables are declared as pvar object. 


%==========================================================================
% Load F/A-18 Physical Parameter data and  airspeed

f18_data; 


% Declare Polynomial Variables ( State / Input)
pvar  p1 r phi alpha beta q
pvar  d_RUD d_AIL d_STAB



%==========================================================================
% Approximated Aerodynamic Model 
% 
% Least Sqaure Estimation has been employed for some aerodynamic
% coefficients.  For more details on how this has been done refer to the 
% folder : 
%         2_F18ReducedModel\NonlinearModel\AeroApprox.m
%
% References: 
%
% (1) Kenneth W. Iliff  and Kon-Sheng C. Wang; 
%     Extraction of lateral-directional stability and  control derivatives 
%     for the basic F-18 aircraft at high angles of attack. 
%     NASA TM-4786 , 1997.
% 
% (2) Marcello R. Napolitano and Alfonso C. Paris and Brad A. Seanor,
%     Estimation of the lateral-directional aerodynamic parameters from 
%     flight data for the NASA F/A-18 HARV.
%     AIAA Atmospheric Flight Mechanics Conference,
%     AIAA-96-3420-CP, 1996, 479-489. 
%
% (3) Marcello R. Napolitano and Alfonso C. Paris and Brad A. Seanor,
%     Estimation of the longitudinal aerodynamic parameters from flight 
%     data for the NASA F/A-18 HARV
%     AIAA Atmospheric Flight Mechanics Conference,
%     AIAA-96-3420-CP, 1996, 469-478.
%
% (4) Marcello R. Napolitano and Joelle M. Spagnuolo,
%     Determination of the stability and control derivatives of the 
%     NASA F/A-18 HARV using flight data, NASA CR-194838, 1993.
%
% (5) Lulch C. Daniel, Analysis of the Out-of-Control Falling Leaf Motion
%     using a Rotational Axis Coordinate System. MS thesis
%     Virginia Polytechnic Institue and State Unniversity, 1998.


% Rolling Moment        
C_l  =  (0.8102*alpha^2 - 0.6446*alpha - 0.0427)*beta ...
        + (-1.5526e-001*alpha + 1.5417e-001)*d_AIL ...
        + (-1.8583e-002*alpha +  1.9433e-002)*d_RUD ...
        + (2.0054e-001*alpha -3.3700e-001)*p1*b/2/V;


    
% Yawing Moment
C_n =   (-3.9165e-001*alpha^2 +  3.6480e-002*alpha +  8.9354e-002)*beta...
       +  (-0.0213*alpha +  0.0051)*d_AIL ...
       +  (5.3361e-002*alpha -7.2422e-002)*d_RUD ...
       +  (-7.1620e-002 *alpha-4.3750e-001)*r*b/2/V;

   
% SideForce
C_Y        = ( 0.5781*alpha^2 +0.2834*alpha -0.8615)*beta...
             + (-4.4857e-001*alpha +  3.0794e-001)*d_RUD ...
             +  ( 4.2702e-001*alpha -1.0472e-001)*d_AIL;

% Pitching Moment
C_m =  (-0.9931*alpha) +  (0.6401*alpha -1.1055) * d_STAB + ...
       ( -14.3*alpha - 2)*c*q/2/V;


% See Aerodynamic Reference #4
% Drag Buildup
Cd  = (0.53594*alpha -3.8841*alpha^2 -1.6387*beta^2) + ....
      ( 0.027584 - 0.77719 *alpha)*d_STAB  ; 


% See Aerodynamic Reference #4
% Lift Forces
CLift  = (-4.8313*alpha + 3.3609*alpha^2) + (-0.85639 + 0.35733*alpha)*d_STAB;


% Roll/Pitch/Yaw Moments and Sideforce Build-up 
qbar =  1/2*rho*V^2;                 % Dynamic pressure
L    =  C_l*qbar*S*b ;
N    =  C_n*qbar*S*b;
M    =  C_m*qbar*S*c;
Y    =  C_Y*qbar*S ;



%========================================================================== 
% Equations of Motion
% This eqautions of motion describe roll-coupled manuever for aircraft. 
%
% Reference : A.A. Schy and M. E. Hannah
%             Prediction of jump phenomena in roll-coupled maneuvers 
%             of airplanes. Journal of Aircraft, 1977, Volume 14; 



betad1 =   ( Y  - qbar*S*Cd*beta)/m/V + g*(phi)/V + p1*alpha - r ;

alphad1 =  q + qbar*S*CLift/m/V  - p1*beta  ;

pd1 = (Izz* L + Ixz* N - ((Ixz^2 + Izz*(Izz - Iyy))*r)*q)/(Ixx*Izz -Ixz^2);
 
qd1 =  ( M  + (Izz -Ixx)*p1*r)/Iyy;

rd1 = (Ixz * L  + Ixx * N + ( (Ixz^2 + Ixx*(Ixx - Iyy))*p1)*q)/(Ixx*Izz -Ixz^2);

phid1 = p1 ; 


x = [beta;p1;r;phi;alpha];
xdot = [betad1; pd1; rd1; phid1; alphad1];

