%
% This file performs least square technique to approximate the
% higher order model upto a cubic model. 
% 
%  Example for the Approximation: 
% 
%  Lets look at the alpha*beta terms in betadot. This can be done as
%   ( e(1) represents the betadot terms without the linear terms) 
% 
%  b2 = collect(e(1),alpha*beta); ( Look at b2 in the following code) 
%
%  b2 =  0.00027226*alpha^4*beta - 0.013118*alpha^3*beta - ....
%        0.0037151*alpha*beta^3 + 0.2662*alpha^2*beta + 0.085341*beta^3 ...
%        - 0.018853*alpha*beta
% 
%  We want to make a cubic approximation for this expression. 
%  Desired Form: (look at Hab in the following code)
% 
%  c1*alpha^3 + c2* beta^3 + c3*alpha*beta^2 + c4*alpha^2*beta + c5*alpha*beta 
%  
%  Now, we can evaluate b2 for different values of alpha and beta within
%  a specfied range of alpha and beta. This will form the "b" vector for
%  "Ax = b" equation.  The "A" matrix can be formed by evaluating the
%  following expression at the same alpha-beta values. 
%
%      A =  [alpha^3  beta^3 alpha*beta^2  alpha^2*beta  alpha*beta] 
%   Now, x :=[c1 c2 c3 c4 c5]' & can be found by solving the following way:
%    
%                       x := A\b; 
% 
% Likewise, other higher order terms have been handled.  
%
% Comments: Some terms may arise as a first order term. For example : the 
% t3p term in the following code is : 
%  [ 0.0011287*alpha^3 - 0.0063618*alpha^2 + 0.1026*alpha - 0.75435*q ]* p
% Here, we will manually add the -0.75435*q*p term in the final formulation
% of the closed loop approximation in the F18Clp_Revised.



%==========================================================================
% Set up the grid for alpha-beta
%
% Number of data points within the specified grid 
N = 10;

% Grid Data
alpha2 =  linspace( 0*d2r,40*d2r,N);   % Alpha grid := 0 to 40 Degree
beta2  =  linspace(-20*d2r,20*d2r,N);  % Beta grid  := -20 to 20 Degree



%==========================================================================
% Construct the Terms : We only need to approximate betadot, pdot, rdot. 

%-----------------------------------
% betadot 
bdot_ab = zeros(N^2,1);  % alpha-beta terms
bdot_ap = zeros(N^2,1);  % alpha-p terms
bdot_ar = zeros(N^2,1);  % alpha-r terms
bdot_axc= zeros(N^2,1);  % alpha-xc terms

b2   = collect(e(1),alpha*beta);
t1p  = jacobian(e(1),p1);
t1r  = jacobian(e(1),r);
t1xc = jacobian(e(1),xc);

%-----------------------------------
% pdot 
pdot_ab= zeros(N^2,1);   % alpha-beta terms
pdot_ap= zeros(N^2,1);   % alpha-p terms
pdot_ar= zeros(N^2,1);   % alpha-r terms
pdot_axc= zeros(N^2,1);  % alpha-xc terms

p2   = collect(e(2),alpha*beta);
t2p  = jacobian(e(2),p1);              
t2r  = jacobian(e(2),r);
t2xc = jacobian(e(2),xc);


%-----------------------------------
% rdot 
rdot_ab= zeros(N^2,1);  % alpha-beta terms
rdot_ap= zeros(N^2,1);  % alpha-p terms
rdot_ar= zeros(N^2,1);  % alpha-r terms
rdot_axc= zeros(N^2,1); % alpha-xc terms

r2   = collect(e(3),alpha*beta);
t3p  = jacobian(e(3),p1);
t3r  = jacobian(e(3),r);
t3xc = jacobian(e(3),xc);

%-----------------------------------
% Index Initialization
l = 0; 

%==========================================================================
% Loop for froming the "b" vector for the above mentioned "Ax = b" problem

for j = 1:N
 for i = 1:N
 
     l = l+1; 
  
    %-----------------------------------
    % betadot 
    bdot_ab(l) = b2.coeff(1).*alpha2(i).^4.*beta2(j) +  b2.coeff(2).*alpha2(i).^3.*beta2(j)...
                +  b2.coeff(3).*alpha2(i).*beta2(j).^3+ b2.coeff(4).*alpha2(i).^2.*beta2(j)+...
               b2.coeff(6).*alpha2(i).*beta2(j)+ b2.coeff(5).*beta2(j).^3;

    bdot_ap(l) = t1p.coeff(1).*alpha2(i).^3 +t1p.coeff(2).*alpha2(i).^2  + t1p.coeff(3).*alpha2(i) ;

    bdot_ar(l) = t1r.coeff(1).*alpha2(i).^3 + t1r.coeff(2).*alpha2(i).^2 + t1r.coeff(3).*alpha2(i);

    bdot_axc(l) = t1xc.coeff(1).*alpha2(i).^3 + t1xc.coeff(2).*alpha2(i).^2 + t1xc.coeff(3).*alpha2(i);


    %-----------------------------------
    % pdot

    pdot_ab(l) = p2.coeff(1).*alpha2(i).^4.*beta2(j) + p2.coeff(2).*alpha2(i).^3.*beta2(j)...
                 + p2.coeff(3).*alpha2(i).*beta2(j).^3 + p2.coeff(4).*alpha2(i).^2.*beta2(j)...
                 + p2.coeff(6).*alpha2(i).*beta2(j)+ p2.coeff(5).*beta2(j).^3;

    pdot_ap(l) = t2p.coeff(1).*alpha2(i).^3 + t2p.coeff(2).*alpha2(i).^2  + t2p.coeff(3).*alpha2(i) ;

    pdot_ar(l) = t2r.coeff(1).*alpha2(i).^3 + t2r.coeff(2).*alpha2(i).^2 + t2r.coeff(3).*alpha2(i);

    pdot_axc(l) = t2xc.coeff(1).*alpha2(i).^3 + t2xc.coeff(2).*alpha2(i).^2 + t2xc.coeff(3).*alpha2(i);


    %-----------------------------------
    % rdot

    rdot_ab(l) = r2.coeff(1).*alpha2(i).^4.*beta2(j) + r2.coeff(2).*alpha2(i).^3.*beta2(j) ...
                + r2.coeff(3).*alpha2(i).*beta2(j).^3 + r2.coeff(4).*alpha2(i).^2.*beta2(j) ...
                + r2.coeff(6).*alpha2(i).*beta2(j)+ r2.coeff(5).*beta2(j).^3;

    rdot_ap(l) = t3p.coeff(1).*alpha2(i).^3 + t3p.coeff(2).*alpha2(i).^2  + t3p.coeff(3).*alpha2(i) ;

    rdot_ar(l) = t3r.coeff(1).*alpha2(i).^3 + t3r.coeff(2).*alpha2(i).^2 + t3r.coeff(3).*alpha2(i);

    rdot_axc(l) = t3xc.coeff(1).*alpha2(i).^3  + t3xc.coeff(2).*alpha2(i).^2 + t3xc.coeff(3).*alpha2(i);


 end
end


%==========================================================================
% Loop for froming the "A" matrix for the above mentioned "Ax = b" problem


p = 0; 


for i = 1 : N
    for j = 1: N 
    
    p = p + 1;
    Hab(p,:) = [alpha2(j)^3  beta2(i)^3  alpha2(j)*beta2(i)^2  alpha2(j)^2*beta2(i)  alpha2(j)*beta2(i) ];
    Ha(p,:) = [alpha2(j)^2  alpha2(j)  1];
    x(p) = alpha2(j);
    y(p) = beta2(i);
    end
end

%==========================================================================
% Solve the Least Square Problem

% beta2dot
HabLSbd = Hab \bdot_ab;
HapLSbd = Ha  \bdot_ap;
HarLSbd = Ha  \bdot_ar;
HaxcLSbd = Ha  \bdot_axc;

% Pdot
HabLSpd = Hab \pdot_ab;
HapLSpd = Ha  \pdot_ap;
HarLSpd = Ha  \pdot_ar;
HaxcLSpd = Ha  \pdot_axc;


% rdot
HabLSrd = Hab \rdot_ab;
HapLSrd = Ha  \rdot_ap;
HarLSrd = Ha  \rdot_ar;
HaxcLSrd = Ha  \rdot_axc;




