%
% This file performs linear robustness comparisons of the two control laws;
% Baseline  vs  Revised.  
% 
% Analyses performed are : 
%
%  (i)  Loopmargin Analysis
%  (ii) Input Multiplicative Uncertainty : 
%                    (a) Full Block Uncertainty
%                    (b) Diagonal Uncertainty
% (iii) Parametric Uncertainty in Aerodynamic Coefficient 
%



%==========================================================================
% Generate Closed Loop Linear Plants for Different Flight Condition

Gen_LinF18ClpPlants; 


%-----------------------------------------------------------------------
% Stack The Stable Closed Loop LTI Models

Pss_BaselinePlants(:,:,1:7) = stack(1, Pss_Baseline_Beta3mAlpha0, ...
                 Pss_Baseline_Beta3Alpha0, Pss_Baseline_Beta0Alpha10,...
                 Pss_Baseline_Beta3Alpha10, Pss_Baseline_Beta0Alpha0,...
                 Pss_Baseline_Beta2mAlpha6, Pss_Baseline_Beta2Alpha6); 
             
Pss_RevisedPlants(:,:,1:7) = stack(1, Pss_Revised_Beta3mAlpha0, ...
                Pss_Revised_Beta3Alpha0, Pss_Revised_Beta0Alpha10,...
                Pss_Revised_Beta3Alpha10, Pss_Revised_Beta0Alpha0,...
                 Pss_Revised_Beta2mAlpha6, Pss_Revised_Beta2Alpha6);
            
%==========================================================================
%                       Linear Analysis
%==========================================================================



%==========================================================================
% Loopmargin Analysis
% 
% Loopmargin Analysis has been performed around Nominal flight condition:
% Alpha = 0 degree; Beta = -3 degree. 
%==========================================================================

Pss_Baseline = Pss_BaselinePlants(:,:,1,1);
Pss_Revised = Pss_RevisedPlants(:,:,1,1);

%--------------------------------------------------------------------------
% Baseline Control Law
[smiB,dmiB,mmiB,smoB,dmoB,mmoB,mmioB]=loopmargin(Pss_Baseline,Css_Baseline);


%--------------------------------------------------------------------------
% Revised Control Law
[smiR,dmiR,mmiR,smoR,dmoR,mmoR,mmioR]=loopmargin(Pss_Revised,Css_Revised);

% Extract the DMPLOT results to plot Disk Margin  
[dgb,dpb] = dmplot(max(mmiB.GainMargin));
[dgr,dpr] = dmplot(max(mmiR.GainMargin)); 


%--------------------------------------------------------------------------
% Plot Results

figure(1)
plot(dgb,dpb,'-r','LineWidth',2)
hold on 
plot(dgr,dpr,'-b','LineWidth',2) 
legend('Baseline','Revised','Location','Best')
grid on 
xlabel('Gain Variation, dB')
ylabel('Phase Variation, deg')


disp('End Loopmargin Analysis')
%==========================================================================


%==========================================================================
% Input Multiplicative Uncertainty Analysis
% (a)  Diagonal Input Multiplicative  Uncertainty 
% (b)  Full Block Input Multiplicative  Uncertainty
%==========================================================================


%==========================================================================
% Diagonal Input Multiplicative  Uncertainty 

d1=ultidyn('d1');
d2=ultidyn('d2');
d3=ultidyn('d3');
DeltaDiag= diag([d1,d2,d3]);


%--------------------------------------------------------------------------
% Frequency shaping of error dynamics

% Frequency Shaping of weighting function.  We have chosen it to be 1. 
W_in = eye(3); 


for i1 = 1:7
    
     Pss_Baseline = Pss_BaselinePlants(:,:,i1,1); 
     Pss_Revised = Pss_RevisedPlants(:,:,i1,1);
      
      
    %--------------------------------------------------------------------------
    % Building up Baseline Uncertain Model : Diagonal

    PuncB = Pss_Baseline*(eye(3)+W_in*DeltaDiag);

    % Uncertain Baseline Closed loop
    CLuncB = feedback(PuncB,Css_Baseline);
    
    % Frequency Response Data (FRD) models
    om = logspace(-2,2,100); 
    CLuncB = frd(CLuncB,om); 

    %--------------------------------------------------------------------------
    % Robust Stability Analysis for Baseline Model : Diagonal
    [STABMARGB,DESTABUNCB,REPORTB,INFOB] = robuststab( CLuncB );
    BaselineMUDiag(:,i1) = INFOB.MussvBnds(1); 

    %--------------------------------------------------------------------------
    % Building up Revised Uncertain Model  : Diagonal  

    PuncR = Pss_Revised*(eye(3)+W_in*DeltaDiag);

    % Uncertain Revised Closed loop
    CLuncR = feedback(PuncR,Css_Revised);

    % Frequency Response Data (FRD) models
    CLuncR = frd(CLuncR,om); 

    %--------------------------------------------------------------------------
    % Robust Stability Analysis for Revised Model : Diagonal
    [STABMARGR,DESTABUNCR,REPORTR,INFOR] = robuststab( CLuncR );
    RevisedMUDiag(:,i1) = INFOR.MussvBnds(1);  

end

%--------------------------------------------------------------------------
% Plot Robustness Results
    
figure(2)
for i1 = 1:7
    
    semilogx(BaselineMUDiag(:,i1),'--r')
    hold on
    semilogx(RevisedMUDiag(:,i1),'-b')
    hold on
    
end

grid on
xlabel('Frequency (rad/s)')
ylabel ('\mu')
title('Robustness Analysis for Diagonal Input Multuplicative Uncertainty')
legend('Baseline ','Revised')



%==========================================================================
% Full Block Input Multiplicative Uncertainty 
        
DeltaFull = ultidyn('Delta',[3 3],'Bound',1);


for i1 = 1:7
    
    Pss_Baseline = Pss_BaselinePlants(:,:,i1,1); 
    Pss_Revised = Pss_RevisedPlants(:,:,i1,1);

    %--------------------------------------------------------------------------
    % Building up Baseline Uncertain Model : Full Block Uncertainty

    PuncB = Pss_Baseline*(eye(3)+W_in*DeltaFull);

    % Uncertain Baseline Closed loop
    CLuncB = feedback(PuncB,Css_Baseline);
    
    % Frequency Response Data (FRD) models
    om = logspace(-2,2,100); 
    CLuncB = frd(CLuncB,om); 

    %--------------------------------------------------------------------------
    % Robust Stability Analysis for Baseline Model: Full Block Uncertainty
    [STABMARGB,DESTABUNCB,REPORTB,INFOB] = robuststab( CLuncB );
    BaselineMUFull(:,i1) = INFOB.MussvBnds(1); 



    %--------------------------------------------------------------------------
    % Building up Revised Uncertain Model  : Full Block Uncertainty  

    PuncR = Pss_Revised*(eye(3)+W_in*DeltaFull);

    % Uncertain Revised Closed loop
    CLuncR = feedback(PuncR,Css_Revised);
    
    % Frequency Response Data (FRD) models
    CLuncR = frd(CLuncR,om); 
    
    %--------------------------------------------------------------------------
    % Robust Stability Analysis for Revised Model: Full Block Uncertainty 
    [STABMARGR,DESTABUNCR,REPORTR,INFOR] = robuststab( CLuncR );
    RevisedMUFull(:,i1) = INFOR.MussvBnds(1);  

end
%--------------------------------------------------------------------------
% Plot Robustness Results
    
figure(3)
for i1 =1:7
    semilogx(BaselineMUFull(:,i1),'--r')
    hold on
    semilogx(RevisedMUFull(:,i1),'-b')
    hold on
   
end
grid on
xlabel('Frequency (rad/s)')
ylabel ('\mu')
title('Robustness Analysis for Full Input Multuplicative Uncertainty')
legend('Baseline ','Revised')

disp('End Input Multiplicative Uncertainty Analysis')

%==========================================================================




%==========================================================================
% Parametric Uncertainty in Aerodynamic Coefficient Analysis
%==========================================================================

    
% Lateral Direction Important Terms

%  Uncertain Entries    Remarks 
%  in A Matrix
%  (1,1)                Sidefore due to sideslip      
%  (2,1)                Rolling moment due to sideslip
%  (3,1)                Yawing moment due to sideslip
%  (3,3)                Yaw Damping
%  (2,2)                Roll Damping

% Longitudinal Direction Important Terms

%  Uncertain Entries    Remarks   
%  in A Matrix
%  (5,6)                Normal fore due to Pitch Rate      
%  (6,5)                Pitch Stiffness
%  (6,6)                Pitch Damping



% Uncertainty Variation in the real parameters
var_unc = 10; 

for i1 = 1:7
    
    Pss_Baseline = Pss_BaselinePlants(:,:,i1,1); 
    Pss_Revised = Pss_RevisedPlants(:,:,i1,1);
    

    %==========================================================================
    % Analysis on  Baseline Model  

    % Declare Uncertain Variable : Real Uncertainty
    d1 = ureal('d1',Pss_Baseline.A(1,1),'pe',var_unc); 
    d2 = ureal('d2',Pss_Baseline.A(2,1),'pe',var_unc);
    d3 = ureal('d3',Pss_Baseline.A(3,1),'pe',var_unc); 
    d4 = ureal('d4',Pss_Baseline.A(3,3),'pe',var_unc);
    d5 = ureal('d5',Pss_Baseline.A(5,6),'pe',var_unc); 
    d6 = ureal('d6',Pss_Baseline.A(6,5),'pe',var_unc); 
    d7 = ureal('d7',Pss_Baseline.A(6,6),'pe',var_unc); 
    d8 = ureal('d8',Pss_Baseline.A(2,2),'pe',var_unc);


    %--------------------------------------------------------------------------
    % Uncertain A Matrix
    Aunc = [ d1 Pss_Baseline.A(1,2:end); ...
            d2 d8 Pss_Baseline.A(2,3:end);...
            d3 Pss_Baseline.A(3,2) d4 Pss_Baseline.A(3,4:end); ...
            Pss_Baseline.A(4,:);...
            Pss_Baseline.A(5,1:5) d5; ...
            Pss_Baseline.A(6,1:4) d6 d7 ]; 

    %--------------------------------------------------------------------------
    % Uncertain Open Loop Baseline Model 
    PuncB  = ss(Aunc,Pss_Baseline.B,Pss_Baseline.C,Pss_Baseline.D);

    %--------------------------------------------------------------------------
    % Uncertain Closed loop
    CLuncB = feedback(PuncB,Css_Baseline);


    %--------------------------------------------------------------------------
    % Robust Stability / Mu Analysis for Baseline Model
    
    % Adding a little amount of complexity to improve lower bound results. 
    CLuncB = complexify(CLuncB,0.05);
    
    % Frequency Response Data (FRD) models
    om = logspace(-2,2,100); 
    CLuncB = frd(CLuncB,om); 

    opt2=robopt('Mussv','m9');
    [SMB , DeUncB, repB, infoB] = robuststab(CLuncB,opt2);
    BaselinePara(:,i1) = infoB.MussvBnds(1);



    %==========================================================================
    % Analysis on Revised Model  

    %--------------------------------------------------------------------------
    % Declare Uncertain Variable : Real Uncertainty
    d1 = ureal('d1',Pss_Revised.A(1,1),'pe',var_unc); 
    d2 = ureal('d2',Pss_Revised.A(2,1),'pe',var_unc); 
    d3 = ureal('d3',Pss_Revised.A(3,1),'pe',var_unc); 
    d4 = ureal('d4',Pss_Revised.A(3,3),'pe',var_unc);
    d5 = ureal('d5',Pss_Revised.A(5,6),'pe',var_unc); 
    d6 = ureal('d6',Pss_Revised.A(6,5),'pe',var_unc); 
    d7 = ureal('d7',Pss_Revised.A(6,6),'pe',var_unc); 
    d8 = ureal('d8',Pss_Revised.A(2,2),'pe',var_unc);


    %--------------------------------------------------------------------------
    % Uncertain A Matrix
    Aunc = [ d1 Pss_Revised.A(1,2:end); ...
            d2 d8 Pss_Revised.A(2,3:end);...
            d3 Pss_Revised.A(3,2) d4 Pss_Revised.A(3,4:end); ...
            Pss_Revised.A(4,:);...
            Pss_Revised.A(5,1:5) d5; ...
            Pss_Revised.A(6,1:4) d6 d7 ]; 

    %--------------------------------------------------------------------------
    % Uncertain Open Loop Revised Model 
    PuncM  = ss(Aunc,Pss_Revised.B,Pss_Revised.C,Pss_Revised.D);

    %--------------------------------------------------------------------------
    % Uncertain  Revised Closed loop
    CLuncM = feedback(PuncM,Css_Revised);


    %--------------------------------------------------------------------------
    % Robust Stability / Mu Analysis
    
    % Adding a little amount of complexity to improve lower bound results. 
    CLuncM = complexify(CLuncM,0.05);

    % Frequency Response Data (FRD) models
    CLuncM = frd(CLuncM,om); 

    opt2=robopt('Mussv','m9');
    [SMM , DeUncM, repM, infoM] = robuststab(CLuncM,opt2);
    RevisedPara(:,i1) = infoM.MussvBnds(1);
end    
%--------------------------------------------------------------------------
% Plot Results

figure(4)
for i1 = 1:7
    semilogx(BaselinePara(:,i1),'-.r')
    hold on
    semilogx(RevisedPara(:,i1),'-b')
    hold on
end
grid on 
xlabel('Frequency (rad/s)')
ylabel ('\mu')
title('Robustness Comparison of Parametric Uncertiany in Aero Coefficients')
legend('Baseline ','Revised')

% End Parametric Uncertainty in Aerodynamic Coefficient Analysis
%==========================================================================
