%  Search4VdotPos.m
%  This file attempts to solve via Monte Carlo search:
%      min  x0'*N*x0
%      subject to:  dV/dt = grad(V)(x)*f(x) >0 
%
%  The file randomly picks initial conditions on the ellipse x0'*N*x0 = g.
%  If dV/dt(x) is > 0 then g is decreased by gfac and the search continues. 


%==========================================================================
% Add ROA Tool Path
%==========================================================================


%==========================================================================
%                               Generate Model

% State Ordering 
% x     = [beta ;      p1      ; r     ;phi        ; alpha     ; q;   xc ];
% f     = [betad;      pd      ; rd    ;phid       ; alphad    ;qd;   xcd];
%==========================================================================

Baseline = 0;       % Set to zero for revised model 


if Baseline 
    load F18BaselineCubic;
else
    load F18RevisedCubic; 
end

 
%==========================================================================
% Initialization for Simulation


% Search parameters
Ntry = 16000;               % Number of random search tries

% Form Candidate Lyapunov function from linearized analysis
Acl = double(jacobian(cleanpoly(f,[],1),x));
P=lyap(Acl',eye(size(Acl)));
V = x'*P*x;
Vdot = jacobian(V,x)*f;

% Form scaling matrix, used to evaluate norm x'*N*x
N = P;  % Scale using Lyap level sets

% Initial cost value
gmax = inf;
gtry = 5;           % Size of the Initial Ellipsoid
gfac = 0.95;        % Decreasing Factor for the Ellipsoid
ns = max(size(x));
x0min = zeros(ns,1);

%======================================================================
%                          Run Monte Carlo Simulation 
%======================================================================

for i1=1:Ntry;
    
    % Find random initial condition and scale so that x0'*N*x0 = gtry
    x0 = randn(ns,1);
    x0(end) = 0;   % XXX Special for the A/C example--set controller state to zero
    scl = sqrt( (x0'*N*x0)/gtry );
    x0 = x0/scl;
    
    % Evaluate Vdot and check sign
    Vd=double(subs(Vdot,char(x),num2cell(x0)));        
    VdPos = Vd>0;
    
    if VdPos
        % dV/dt>0
        fprintf('Simulation Step=%d \t gtry = %4.8f\n',i1,gtry);
        x0min = x0;
        gmax = gtry;
        gtry = gfac*gtry;
    end
end
    
%-----------------------------------------------------------------------
%   Verify that x0min causes dV/dt>0
%-----------------------------------------------------------------------
[xcl x0min*r2d]
subs( V , char(x), num2cell(x0min) )
subs( Vdot , char(x), num2cell(x0min) )
