%------------------------------------------------------------------
% Coutinho example 3 with VS Iteration Initialized with Vlin
%   Compute an estimate of the region of attraction for Coutinho
%   example 3 using the V-S iteration.  The iteration is initialized
%   with the Lyapunov function from linearized analysis.
%
% References: 
%  1) D.F. Coutinho, C.E. de Souza, and A. Trofino, Stability 
%  Analysis of Implicit Polynomial Systems, vol 54, no 5, IEEE TAC, 
%  pp. 1012-1018, May 2009.
%------------------------------------------------------------------
clear all; close all;

% Create vector field
pvar x1 x2;
x = [x1;x2];
f = [(x2^2-1)*x1 ; (x1^2-1)*x2 + (1-x2^2)*x1];

% Create shape function and vector of Lyapunov function monomials
Vdeg = 2;
M = randn(2,2); M = M'*M;
M = eye(2);
p = x'*M*x;
zV = monomials(x,2:Vdeg);    
z1maxd = ceil((Vdeg-p.maxdeg)/2);
z1 = monomials(x, 0:z1maxd ); 
z2 = monomials(x, 1:2 );
L2 = 1e-6*(x'*x);

% Construct Lyap function from linearization 
Vlin=linstab(f,x);

% Run V-s iteration
Nsteps = 10; 
ph = [];
opts = [];
opts.gmin = 0; 
opts.gmax = 50;
opts.L2 = L2;
for i1=1:Nsteps;    
    if i1==1
        V = Vlin;        
    else
        % roavstep
        [V,c] = roavstep(f,p,x,zV,b,g,s1,s2,opts);
        
        % Scale V by previous gamma.  This roughly normalizes V
        V = V/g;
    end
    
    % get gamma
    [gbnds,s2]=pcontain(jacobian(V,x)*f+L2,V,z2,opts);
    g = gbnds(1);

    % get beta
    [bbnds,s1]=pcontain(V-g,p,z1,opts);
    b = bbnds(1);
    
    fprintf('i1 = %d  \t beta = %4.3f\n',i1,b);

    % Plot V/p contours for ROA estimate 
    if  ~isempty(ph)
        delete(ph)
    end
    domain = [-1.5 1.5 -1.5 1.5];
    [C,ph(1)]=pcontour(V,g,domain,'r'); hold on;
    [C,ph(2)]=pcontour(p,b,domain,'b'); hold off;
    title(['Iteration = ' int2str(i1) ' beta = ' num2str(b)]);
    axis(domain)
    drawnow;    
end
hold on
axis([-1.5 1.5 -1.5 1.5])
plot(1,1,'o',1,-1,'o',-1,1,'o',-1,-1,'o')
hold off;

fprintf('Volume of {x : V(x)<=g} = %4.3f*pi\n',pvolume(V,g)/pi);

