%------------------------------------------------------------------
% Benchmark example with VS Iteration initialized with Vlin
%   Compute an estimate of the region of attraction for benchmark
%   example given in Section 3.4 of W. Tan's Thesis. The V-S iteration
%   is initialized with the Lyapunov function from linearized analysis.
%   The ROA for this example is, by construction, a known ellipsoid.
%
% References: 
%  1) W. Tan.  Nonlinear Control Analysis and Synthesis using
%     Sum-of-Squares Programming. Ph.D. Thesis, Univ. of California,
%     Berkeley, Spring 2006.
%     (See Section 3.4 for results on the benchmark example)
%
%  2) E.J. Davison and E.M. Kurak. A computational method for determining
%      quadratic Lyapunov functions for non-linear systems.  Automatica,
%      7:627-636, 1971.
%------------------------------------------------------------------
clear all; close all;

% Create vector field
% The ROA is given by { x: x'*B*x<1}
nx = 3;  
x=mpvar('x',nx,1);

[UU,SS,VV] = svd(randn(nx));
L = exp(2*randn(nx,1));
B = UU*diag( L )*UU';

xdot = -x + (x'*B*x)*x;
f = xdot;

% Create shape function and vector of Lyapunov function monomials
[UU,SS,VV] = svd(randn(nx));
L = exp(2*randn(nx,1));
R = UU*diag( L )*UU';
p = x'*R*x;

Vdeg = 4;
zV = monomials(x,2:Vdeg);    
z1maxd = ceil((Vdeg-p.maxdeg)/2);
z1 = monomials(x, 0:z1maxd ); 
z2 = monomials(x, 1:2 );
L2 = 1e-6*(x'*x);

% Compute true value of beta 
% (See Sec 3.4 of W. Tan for derivation)
lmax = max(eig(B,R));
b_true = 1/lmax;

% Construct Lyap function from linearization 
Vlin=linstab(f,x);

% Run V-s iteration
Nsteps = 5; 
ph = [];
opts = [];
opts.gmin = 0; 
opts.gmax = 50;
opts.L2 = L2;
for i1=1:Nsteps;    
    if i1==1
        V = Vlin;        
    else
        % roavstep
        [V,c] = roavstep(f,p,x,zV,b,g,s1,s2,opts);
        
        % Scale V by previous gamma.  This roughly normalizes V
        V = V/g;
    end
    
    % get gamma
    [gbnds,s2]=pcontain(jacobian(V,x)*f+L2,V,z2,opts);
    g = gbnds(1);

    % get beta
    [bbnds,s1]=pcontain(V-g,p,z1,opts);
    b = bbnds(1);
    
    fprintf('i1 = %d  \t beta = %4.3f \t beta_true = %4.3f\t',i1,b,b_true);
    fprintf('beta/beta_true = %4.3f\n',b/b_true);
end

