function [c,info,ep] = findVfeas_New(fList,A,b,constr,opt)
% function [c,info] = findVfeas(fList,A,b,constr,opt)
%
% DESCRIPTION 
%   Solves the feasibility problem for a candidate Lyap. function:
%    Find c  
%    s.t. A*c<=b
%         V(x,c) - L1(x) is SOS
%         Quadratic[-d(V(x,c))/dx*f_i(x)+mu*V(x,c)] is SOS (i=1,..,Nsys)
%    where V(x,c) = Z(x)'*c and f_i(x) is the i^th vector
%    field in fList.
%   
% INPUTS 
%   fList: 
%   A:
%   b:
%   constr:
%   opt: 
%
% OUTPUTS 
%   c:
%   info: 
%
% SYNTAX 
%   [c,info] = findVfeas(fList,A,b,constr,opt)
%
% REFERENCE
%
% 1/31/08  PJS  Updating to use sosp 
%

% Set usesosp ==1 to use sosp functions. Otherwise this
% will call the code which directly sets up the
% problem in Sedumi
usesosp = opt.usesosp;
usesosp = 1;

if usesosp
    %---------------------------------------------------------------------
    % Get Sedumi options
    %---------------------------------------------------------------------
    sedopts = opt.sedopts;
    sosopts = sedopts;
    
    %---------------------------------------------------------------------
    % Get constraint options
    %---------------------------------------------------------------------
    Z = constr.zV;
    L1 = constr.L1;
    L2 = constr.L2;
    L2LP = constr.L2LP;
    mu = constr.mu;
    x = fList.var;

    %---------------------------------------------------------------------
    % Formulate SOS constraints
    %---------------------------------------------------------------------
    % XXX PJS 5/21/2009 Updated to new mpvar syntax
    %cV = mpvar( length(Z), 1, 'c');
    cV = mpvar( 'c', length(Z), 1);
    if ~opt.feasonly pvar ep; end
    V = Z'*cV;
    sosconstr(1) = V-L1;

    Vquad = cleanpoly(Z,[],2)'*cV;
    Nsys = size(fList,2);
    for i0 = 1:Nsys
        flin = cleanpoly( fList(:,i0), [], 1);

        if any(flin~=0) && (Vquad~=0)
            % Compute -d(Vquad(x))/dx*flin - mu*Vquad(x)
            if opt.feasonly
                Vd = -(jacobian(Vquad,x)*flin) - mu*Vquad - L2;
            else
                Vd = -(jacobian(Vquad,x)*flin) - mu*Vquad - 2*ep*L2LP; 
            end
            sosconstr = [sosconstr; Vd];
        end
    end
    if ~opt.feasonly
        sosconstr = [sosconstr; ep*L2LP-L2];
    end

    %---------------------------------------------------------------------
    % Formulate LP constraints
    %---------------------------------------------------------------------
    lpconstr.A = A;
    lpconstr.b = b;
    if opt.feasonly
        lpconstr.d = cV;
    else
        lpconstr.d = [cV;ep];
    end

    %---------------------------------------------------------------------
    % Solve SOS Feasibility Problem
    %---------------------------------------------------------------------
    
%     sosopts.solver = 'sedumi';  sosopts.form = 'image';
%     sol=sosp(sosconstr,x,-lpconstr.c(end),lpconstr,sedopts);

    if opt.feasonly
        [dopt,sossol,solinfo]=sosopt(sosconstr,x,[],lpconstr,sosopts);
    else
        [dopt,sossol,solinfo]=sosopt(sosconstr,x,-lpconstr.d(end),lpconstr,sosopts);
    end
    
    %---------------------------------------------------------------------
    % Check feasibility of result
    %---------------------------------------------------------------------    
%     isfeasp = double(check_feasp(sol.sedumi.info,'sedumi')==1);
    isfeasp = double(check_feasp(solinfo.solverinfo,'sedumi')==1);
    
    if isfeasp
        if opt.feasonly
            c = subs(cV,dopt(:,1),dopt(:,2));
        else
            c = subs(cV,dopt(1:end-1,1),dopt(1:end-1,2) );
            ep = subs(ep,dopt(end,1),dopt(end,2) ); 
        end
    else
        c = []; ep = [];
    end
    info.dopt = dopt;
    info.sossol = sossol;
    info.solinfo = solinfo;

    return;
    
else

    %---------------------------------------------------------------------
    % Get algorithm options
    %---------------------------------------------------------------------
    sedopts = opt.sedopts;

    %---------------------------------------------------------------------
    % Get constraint options
    %---------------------------------------------------------------------
    Z = constr.zV;
    L1 = constr.L1;
    L2 = constr.L2;
    mu = constr.mu;

    %---------------------------------------------------------------------
    % Define the LP constraint: A*c<=b
    % To put in Sedumi primal form, introduce a slack variable y 
    % such that  A*c<=b <===> A*c+y=b, y>=0 
    %---------------------------------------------------------------------
    [n_constraint,n_coef] = size(A);
    n_x = length(Z.var);

    K.f = n_coef;               % # of free variables c
    K.l = n_constraint;         % # of non-negative variables y

    % Sedumi requires the primal variables in the order: free variables, 
    % non-negative variabless, SDP variables.  Thus we rewrite A*c+y=b as 
    % As*v=bs where v=[c;y].  v will be appended with SDP vars below
    As=sparse([A speye(K.l)]); 
    bs=sparse(b);

    %---------------------------------------------------------------------
    % Define the constraint: Z(x)*c - L1(x) is SOS 
    %---------------------------------------------------------------------

    % Create constraint matrices
    [w,b,Aq,Ac] = gramconstraint(-L1,Z');  

    % Add constraints
    K.s = length(w);       % Dimension of Q
    szAs = size(As,1);
    As = blkdiag(As,Aq);
    szAc = size(Ac);
    As(szAs+(1:szAc(1)),1:szAc(2)) = Ac;
    bs = [bs; b];

    %---------------------------------------------------------------------
    % Define constraint: 
    %   Quadratic part of -d(V(x))/dx*fList(:,i)+mu*V(x) -L2(x) is SOS
    % where V(x) = Z(x)*c
    %---------------------------------------------------------------------

    % Find Quadratic terms of Z
    Zquad = cleanpoly(Z,[],2);

    % Add one constraint for each vector field in fList
    Nsys = size(fList,2);
    wd = cell(Nsys,1);
    for i0=1:Nsys
        % Find linear terms of f 
        flin = cleanpoly( fList(:,i0), [], 1);

        if any(flin~=0) && any(Zquad~=0)
            % Compute Zd(x) such that:
            %   Zd(x)*c = -d(Zquad(x)*c)/dx*flin +mu*Zquad(x)'*c
            Zd = -(jacobian(Zquad')*flin)' + mu*Zquad';

            % Create constraint matrices
            [w,b,Aq,Ac] = gramconstraint(-L2,Zd);  
            wd{i0} = w;

            % Add constraints 
            K.s = [K.s length(w)];       % Dimension of Q        
            szAs = size(As,1);
            As = blkdiag(As,Aq);
            szAc = size(Ac);
            As(szAs+(1:szAc(1)),1:szAc(2)) = Ac;
            bs = [bs; b];
        end    

    end

    %---------------------------------------------------------------------
    % Solve feasibility problem in Sedumi Primal Form
    % Find v = [c;y; Q(:); Qd1(:); ... ; QdN(:)] such that
    %    As*v  = bs
    %    v in K
    %---------------------------------------------------------------------
    [v,vd,sedumi_info] = sedumi(As,bs,[],K,sedopts); 
    

    % Return if problem is infeasible 
    isfeasp = double(check_feasp(sedumi_info,'sedumi')==1);
    if ~isfeasp
        c = [];
        info.w = [];
        info.Q = [];
        info.wd = [];
        info.Qd = [];
        info.y = [];
        info.sedumi = sedumi_info;
        return
    end

    % Clean up 
    tol = 10*sedopts.eps;
    v = v.* ( abs(v)>tol );
    c = v(1:K.f);
    y = v(K.f+1:K.f+K.l);
    Q = mat(v(K.f+K.l+1:K.f+K.l+K.s(1)^2));
    for i1=1:Nsys
        idx = K.f+K.l+sum(K.s(1:i1).^2);
        Qd{i1} = mat( v(idx+1 : idx+K.s(i1+1)^2) );
    end

    % Store info
    info.w = w;
    info.Q = Q;
    info.wd = [];
    info.Qd = Qd;
    info.y = y;
    info.sedumi = sedumi_info;

end
