function [A,b,b1] = traj2LP(simdata, fList, XList, Beta, roaconstr, opt)

%
% DESCRIPTION 
%   Uses simulation data from stable, converging trajectories to 
%   generate the linear programming constraints which describe 
%   Lyapunov function conditions along the trajectories.
%   
% INPUTS 
%   simdata:
%   fList: 
%   XList: 
%   Beta: 
%   roaconstr:
%   opt: 
%
% OUTPUTS 
%   A:
%   b:
%
% SYNTAX 
%   [A,b] = traj2LP(simdata, fList, Beta, roaconstr, opt)
%
% REFERENCE
%   Topcu, Packard, and Seiler,  "Local Stability Analysis Using 
%   Simulations and Sum-of-Squares Programming," to appear in Automatica.
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%


% 3/29/08 UT - everythign related to sampleForPositive has to be
% re-checked.


%-------------------------------------------------
% Set Default Options
%-------------------------------------------------
zV = roaconstr.zV;
p = roaconstr.p;
%--------------------------------------------
% % To see the effect of having the Lyap function from quadratic analysis as
% % shape factor   

% ['traj2lp1']
% p = XList'*opt.pM*XList;
% p = XList'*[ 1.0908   -0.2195
%    -0.2195    1.2205]*XList;
% ['traj2lp2']

mu = roaconstr.mu;
L2 = roaconstr.L2;
L1 = roaconstr.L1;
L2LP = roaconstr.L2LP;

useVdot = opt.useVdot;
useVdecr = opt.useVdecr;
sampleForPositive = opt.sampleForPositive;
timesSampleLMI = opt.timesSampleLMI;
radiusSampleLMI = opt.radiusSampleLMI;
decimN = opt.decimN;

%-------------------------------------------------
% Decimate Simulation Data
%-------------------------------------------------
[simdataDecim] = simdecim(simdata, p, Beta, decimN);

%-------------------------------------------------------------
% Compute the number of constraints and initialize matrices
%-------------------------------------------------------------
Nsys = size(fList,2);

Nconst = 0;

if sampleForPositive
    mindeg = floor(zV.mindeg/2);
    maxdeg = ceil(zV.maxdeg/2);
    zV2 = monomials(XList, mindeg:maxdeg);
    zV2 = zV2.*zV2;
    Nconst = Nconst + length(zV2);
    Nconst = Nconst + timesSampleLMI;
end


for i1 = 1:Nsys
    % Compute number of constraints from stable trajectories
    stab = simdataDecim(i1).stab;
    Nst = length(stab);
    for i2 = 1:Nst
        if useVdot 
            Nconst = Nconst + length(stab(i2).time);
        end
        if useVdecr
           Nconst = Nconst + length(stab(i2).time)-1;
        end
        Nconst = Nconst + 1;
        if sampleForPositive
            Nconst = Nconst + 1;
        end
    end
    if sampleForPositive
        Nconst = Nconst + timesSampleLMI;
    end
   %% Compute number of constraints from unstable trajectories
    Nunst = length(simdataDecim(i1).unstab);
    Nconst = Nconst+Nunst;
end
A = zeros(Nconst,length(zV)+1);
b = zeros(Nconst,1);
b1 = zeros(Nconst,1);

%-------------------------------------------------------------
% Form LP Constraints
%-------------------------------------------------------------
cnt = 1;

if sampleForPositive
    [vec,e] = poly2basis(zV2',zV);
    [vecL1,e] = poly2basis(L1,zV);
    sz = size(vec,2);
    A(1:sz,:) = (-vec + repmat(vecL1,1,sz))';
    b(1:sz) = zeros(sz,1);
    b1(1:sz) = zeros(sz,1);
    cnt = cnt + sz;
    
    % XXX PJS 5/20/2009 Updated to psample
    %[xin, xon] = polyset_sample(XList'*XList,radiusSampleLMI^2,timesSampleLMI/2,zeros(size(XList)));
    [xin, xon] = psample(XList'*XList-radiusSampleLMI^2,XList,zeros(size(XList)),timesSampleLMI/2);
    xon = [xon -xon];
    
    %% Sample V-l1 > 0
    Veval = peval(xon, zV.coef, zV.deg)';
    L1eval = peval(xon,L1.coef,L1.deg)';
    A(cnt:cnt+timesSampleLMI-1,:) = [-Veval zeros(size(Veval,1),1)];
    b(cnt:cnt+timesSampleLMI-1,:) = -L1eval;
    b1(cnt:cnt+timesSampleLMI-1,:) = -L1eval;
    cnt = cnt + timesSampleLMI;
end
    

for i1 = 1:Nsys
    
     %% Sample the LMI V2dot+mu*V2 <= -l2
    if sampleForPositive
        zV2 = monomials(XList, 2:2);
        [vec,e] = poly2basis(zV2',zV);
        vec = full(vec);
        vecP = polynomial(zeros(size(vec)));
        for i10 = 1:size(vec,2)
            vecP(:,i10) = vec(:,i10).*zV;
        end
        zV2 = sum(vecP,2);
        zV2d = jacobian(zV2, XList)*fList(:,i1);
        z2 = zV2d + mu*zV2;
        V2deval = peval(xon, z2.coef, z2.deg)';
        L2eval = peval(xon,L2.coef,L2.deg)';
        L2LPeval = peval(xon,L2LP.coef,L2LP.deg)';
        A(cnt:cnt+timesSampleLMI-1,:) = [V2deval L2LPeval];
	    b(cnt:cnt+timesSampleLMI-1,:) = zeros(size(L2LPeval));
        b1(cnt:cnt+timesSampleLMI-1,:) = -L2eval;
        cnt = cnt + timesSampleLMI;
    end
    
    stab = simdataDecim(i1).stab;
    Nst = length(stab);
    
    zVd = jacobian(zV, XList)*fList(:,i1);
    z = zVd + mu*zV;
    for i2 = 1:Nst
        time = stab(i2).time;
        ntime = length(time);
        state = stab(i2).state;
        Veval = peval(state, zV.coef, zV.deg)';
        Vdeval = peval(state, z.coef, z.deg)';
        L2LPeval = peval(state,L2LP.coef,L2LP.deg)';
        L2eval = peval(state,L2.coef,L2.deg)';

        %% V(x(1)) <= 1
        A(cnt,:) = [Veval(1,:) 0];
        b(cnt) = 1;
        b1(cnt) = 1;
        cnt = cnt + 1;
        
        if sampleForPositive
            b(cnt) = -peval(state(:,end),L1.coef,L1.deg)';
            A(cnt,:) = [-Veval(end,:) 0];
            cnt = cnt + 1;
        end
        
        %% (Vdot + mu*V) <= -L2
        if useVdot
            A(cnt:cnt+ntime-1,:) = [Vdeval L2LPeval];
            b(cnt:cnt+ntime-1,:) = zeros(size(L2LPeval)); 
            b1(cnt:cnt+ntime-1,:) = -L2eval;
            cnt = cnt + ntime;
        end

        %% V(x(k+1)) - V(x(k)) <= 0
        if useVdecr
            aux = diff(Veval);
            A(cnt:cnt+ntime-2,:) = [aux zeros(size(sux,1),1)];
            b(cnt:cnt+ntime-2) = 0;
            cnt = cnt + ntime - 1;
        end
    end
    
    unstab = simdataDecim(i1).unstab;
    Nunst = length(unstab);
    for i2 = 1:Nunst
        time = unstab(i2).time;
        ntime = length(time);
        state = unstab(i2).state;  
        idxB = unstab(i2).idxsmallestp;
        if ntime

            Veval = peval(state(:,idxB), zV.coef, zV.deg)';

            %% V(x(1)) >= 1
            A(cnt,:) = [-Veval 0];
            b(cnt) = -1;
            b1(cnt) = -1;
            cnt = cnt + 1;
        end
    end
end


