function [pdv,D,Z] = polydecvar(dstr,Z,type)
% function [p,D,Z] = polydecvar(dstr,Z,type)
%
% DESCRIPTION 
%   This function creates a polynomial decision variable, p, for use in
%   SOS optimization problems.  There are two types:
%      1) type='vec': p has the form p = D'*Z where D is a column 
%          vector of decision variable coefficients.
%      2) type='mat': p has the form p = Z'*D*Z where D is a symmetric
%          matrix of decision variable coefficients.
%
%   Note: For the most efficient implementation of SOS constraints, the
%   'mat' type should be used only if p will be constrained to be SOS.
%   In this case, p must be added to the list of SOS constraints that are
%   passed to sosopt.  For any other polynomial decision variables,
%   the 'vec' type should be used.
%
% INPUTS 
%   dstr: Character string to be used in creating the decision variable
%        coefficients.
%   Z: Nz-by-1 column vector of monomials
%   type:  String, either 'vec' or 'mat', that specifies the decision 
%       variable type.  (default: type = 'vec')
%
% OUTPUTS
%   p: polynomial decision variable object.  p has all the properties
%        of a polynomial object but it has additional fields: 'decvars',
%        'monomials', and 'type' where  s.decvars = D, s.monomials = Z,
%        and s.type = type.
%   D: If type='vec' then D is an Nz-by-1 column vector of pvars with
%        names specified by dstr, e.g. if dstr = 'd' then the i^th entry
%        of D is d_i_1. If type='mat' then D is an Nz-by-Nz symmetric
%        matrix of pvars whose (i,j)^th entry is d_i_j for j>i.
%   Z: Nz-by-1 column vector of monomials. This is the same as the input
%        vector Z.
%
% SYNTAX
%   p = polydecvar(dstr,Z)
%   p = polydecvar(dstr,Z,type)
%   [p,D,Z] = polydecvar(dstr,Z,type)
%
% See also: sosopt

%  PJS 4/25/2009  Initial Coding
%  PJS 5/29/2009  Modification for a polydecvar class

superiorto('polynomial');

if nargin == 0
    p = polynomial;
    pdv = [];
    pdv.decvars = [];
    pdv.monomials = [];
    pdv.type = [];
    pdv = class(pdv,'polydecvar',p);    
    return
elseif nargin==2
    type = 'vec';
elseif nargin~=3
    error('Syntax: [p,D,Z] = polydecvar(dstr,Z,type)')
end
Z = polynomial(Z);
if ~isa(Z,'polynomial') || ~ismonom(Z)   
    error('Second input must be a vector of monomials')
end

if strcmp(type,'vec');
    % Create poly dec var using coefficient vector form

    % Get Z degmat with same ordering as the monomials appear in Z 
    Z = polynomial(Z(:));
    Zdeg = Z.coefficient'*Z.degmat;
    lz = length(Z);
            
    % Create coefficient vector
    D = mpvar(dstr,lz,1);
    Dvar = D.varname;
        
    % Create polynomial
    pcoef = sparse(ones(lz,1));
    pdeg = [speye(lz) Zdeg];
    pvarname = [Dvar; Z.var];
    chkval = 0; % skip validity check
    p = polynomial(pcoef,pdeg,pvarname,[1 1],chkval);        
    
elseif strcmp(type,'mat');
    % Create poly dec var using Gram Matrix form

    % Get Z degmat with same ordering as the monomials appear in Z 
    Z = polynomial(Z(:));
    Zdeg = Z.coefficient'*Z.degmat;
    lz = length(Z);
    
    % Create (symmetric) coefficient matrix
    D = mpvar(dstr,lz,lz,'s');
    Dvar = char(D(:));
    
    % Create polynomial and combine terms due to symmetry
    pcoef = sparse( ones(lz^2,1) );

    nxvar = size(Zdeg,2);
    pdeg = repmat(Zdeg,[lz 1]) + ...
        reshape(repmat(Zdeg(:)',[lz 1]),[lz^2 nxvar]);    
    pdeg = [speye(lz^2) pdeg];
    
    pvarname = [Dvar(:); Z.var];
    p = polynomial(pcoef,pdeg,pvarname,[1 1]);
    p = combine(p);
else
    error('Type must be either ''vec'' or ''mat''.')
end

% Instantiate polydecvar class object (derived from polynomial class)
pdv = [];
pdv.decvars = D;
pdv.monomials = Z;
pdv.type = type;
pdv = class(pdv,'polydecvar',p);    
