function feas=checkfeas(info,sossol,feastol)
% function feas=checkfeas(info,sossol,feastol)
%
% DESCRIPTION 
%   This function checks the feasibility of the solution of an SOS
%   optimization.  One or both of two feasibility checks can be performed:
%      1) Check the solver information contained in info for feasibility. 
%      2) Check the Gram matrix decomposition in sossol for feasibility.
%
% INPUTS 
%   info: Solution information structure returned by sossopt.
%   sossol: Solution cell array returned by sosopt.  
%   feastol: Feasibility tolerance used in full feasibility check.
%
% OUTPUTS
%   feas: feas=1 if the solution is feasible and 0 otherwise.
%
% SYNTAX
%   feas=checkfeas(info)
%   feas=checkfeas(sossol)
%   feas=checkfeas(info,sossol)
%
%   feas=checkfeas(info,feastol)
%   feas=checkfeas(sossol,feastol)
%   feas=checkfeas(info,sossol,feastol)
%
% See also sosopt

%  PJS 4/25/2009  Initial Coding
%  PJS 4/30/2009  Added 'full' feasibility check

% Parse inputs
if nargin==1
    if iscell(info)
        sossol = info;        
        chk1 = 0;
        chk2 = 1;
    else
        chk1 = 1;
        chk2 = 0;
    end   
    feastol  = [];
elseif nargin==2
    if isa(sossol,'double')
        feastol = sossol;
        if iscell(info)
            sossol = info;        
            chk1 = 0;
            chk2 = 1;
        else
            chk1 = 1;
            chk2 = 0;
        end   
    else
        chk1 = 1;
        chk2 = 1;    
        feastol = [];
    end
elseif nargin>3
    error('Only one or two input arguments are allowed');
end

% Set default feastol for full feas check
if isempty(feastol)
    feastol = 1e-6;
end

%-------- Feasibility check on solver information
feas1 = 1;
if chk1 == 1
    
    % Get solver type and info returned by solver
    opts = info.opts;
    solverinfo = info.sdpsol.solverinfo;
    solver = opts.solver;

    % Check solution for feasibility
    if strcmpi(solver,'sedumi');
        if  solverinfo.numerr ~= 2 && solverinfo.pinf == 0 && ...
                solverinfo.dinf==0 && solverinfo.feasratio > 0.70 ...
                && (solverinfo.numerr ~= 1 || solverinfo.feasratio>0.9)
           feas1 = 1;
        else
           feas1 = 0;
        end    
    elseif strcmpi(solver,'dsdp')
        if strcmp(solverinfo.stype,'PDFeasible') && solverinfo.stopcode==1
            feas1 = 1;
        else
            feas1 = 0;
        end    
    elseif strcmpi(solver,'sdpam');
        if strcmp(solverinfo.phasevalue,'pdOPT')
            feas1 = 1;
        else
            feas1 = 0;
        end    
    elseif strcmpi(solver,'csdp');
        if solverinfo ==0 % || solverinfo==1 || solverinfo==2  || solverinfo ==3
            feas1 = 1;
        else
            feas1 = 0;
        end
    else
        feas1 = 0;
    end
end

%-------- Feasibility check on Gram matrix decomposition
feas2 = 1;
%feastol = 1e-6;   % XXX feastol should be an option
if chk2 == 1

    % Check feasibility of solution
    N = size(sossol,1);
    d = zeros(N,2);
    for i1=1:N
        p = sossol{i1,1};
        w = sossol{i1,2};
        Q = sossol{i1,3};
        e = p - LOCALwQw(w,Q);
        
        % Project the coeffs of p and e onto a common set of monomials        
        M = monomials([p;e]);
        cp = abs(poly2basis(p,M));
        ce = abs(poly2basis(e,M));
        
        % Check equality: p=w'*Q*w
        idx1 = find(cp<=feastol);
        tmp1 = ce(idx1);
        
        idx2 = find(cp>feastol);
        tmp2 = ce(idx2)./cp(idx2);
        
        d(i1,1) = max( [tmp1(:); tmp2(:)] );
        
        % Check Q >= 0
        reigQ=real(eig(Q));
        mine=min( reigQ );
        maxe=max( reigQ );
        d(i1,2) = -mine/max(maxe,eps);
    end                

    if all(d(:)<=feastol)
        feas2 = 1;
    else
        feas2 = 0;
    end
end


%-------- Total Feasibility
feas = feas1 & feas2;

% if chk1 && chk2 && (feas1~=feas2)
%     warning('Disagreement in Feasibility -- Investigate the cause')
%     keyboard
% end



%-----------------------------------------------------------
% Local Function: 
%   Fast implementation to perform  p = w'*Q*w
%   (This is now only slightly faster mtimes)
%-----------------------------------------------------------
function p = LOCALwQw(w,Q) 

% Get w degmat with same ordering as the monomials appear in w 
% XXX We are assuming that coef mat is a permutation of Identity
w = polynomial(w(:));
wdeg = w.coefficient'*w.degmat;
lw = length(w);
    
% Create a lower triangular Qlt such that w'*Qlt*w = w'*Q*w
% [This will allow us to express the degmat with half the number of terms]
Qlt = tril(Q)+triu(Q,+1)';

% Create coefficient vector 
pcoef = Qlt(:);
idx = find(Qlt~=0);
pcoef = pcoef(idx);

% Compute degmat for p = w*Q*w    
nxvar = size(wdeg,2);
pdeg = repmat(wdeg,[lw 1]) +reshape(repmat(wdeg(:)',[lw 1]),[lw^2 nxvar]);    
pdeg = pdeg(idx,:);

% Create polynomial 
pvarname = w.var;
p = polynomial(pcoef,pdeg,pvarname,[1 1]);
p = combine(p);
