function [w,b,Aq,Ac,Rbasis] = gramconstraint(p,g);  
% function [w,b,Aq,Ac,R] = gramconstraint(p,g);  
%
% DESCRIPTION 
%   Create matrices which can be used to enforce the Gram matrix
%   decomposition of a polynomial.  Given a 1-by-1 polynomial p and
%   a 1-by-N polynomial g then the following Gram matrix decomposition
%   represents a set of linear constraints on the unknown coefficients
%   c and the unknown entries of the matrix Q:
%           p(x)+g(x)*c = w(x)'*Q*w(x)
%   This function computes monomial vectors w and R, matrics Ac and Aq,
%   and a vector b such that
%         p(x) = R(x)'*b  
%         g(x) = R(x)'*Ac
%         w(x)'*Q*w(x) = R(x)'*Aq*Q(:)
%   Thus the polynomial constraint p(x)+g(x)*c = w(x)'*Q*w(x) can
%   be equivalently expressed as the matrix equation:
%         b + Ac*c = Aq*Q(:)  
%
% INPUTS 
%   p: 1-by-1 polynomial
%   g: 1-by-N polynomial [Optional]
%
% OUTPUTS 
%   w: monomials vector used in Gram matrix constraint
%   b, Aq, Ac: b is a vector and Aq,Ac are two matrices such that the
%     matrix equation Aq*Q(:) + Ac*c = b is equivalent to the polynomial
%     constraint equation p(x)+g(x)*c = w(x)'*Q*w(x)
%   R [Optional]: lr-by-1 basis of monomials. [ Default: R=monomials(p) ]
%
% SYNTAX 
%   [w,b,Aq,R] = gramconstraint(p);  
%   [w,b,Aq,Ac,R] = gramconstraint(p,g);  
%
% EXAMPLE
%   pvar x1 x2 
%   p = 5*x1+3*x1*x2-4*x2^2*x1;
%   g = [x1^2-9, 6*x2*x1];
%   [w,b,Aq,Ac,R] = gramconstraint(p,g);  
%
%   lw = length(w);
%   p-R'*b
%   g-R'*Ac
%   Q = [0 2.5 0 1.5; 2.5 0 0 0; 0 0 0 -2; 1.5 0 -2 0];
%   w'*Q*w - R'*Aq*Q(:)

% 1/28/08  PJS     Initial Coding

if nargin < 2
    g = [];
end

%----------------------------------------------------------------------
% Step 1) Build monomials vector needed to write constraint in terms
%   of the the Gram matrix: 
%           p(x) + g(x)*c = w(x)'*Q*w(x)
%   XXX-Prune w(x) with convex hull algo?
%----------------------------------------------------------------------
h = [p g];
mxdg = max(h.maxdeg)/2;
mndg = min(h.mindeg)/2;
w = monomials( h.nvar , [floor(mndg):ceil(mxdg)]);

% Discard monomials 
lw = size(w,1);
mxdg = ceil(max(h.degmat,[],1)/2);
[ridx,cidx] = find( repmat(mxdg,lw,1) < w );
w(ridx,:) = [];

lw = size(w,1);
mndg = floor(min(h.degmat,[],1)/2);
[ridx,cidx] = find( w< repmat(mndg,lw,1) );
w(ridx,:) = [];

lw = size(w,1);
chkval = 0; % skip validity check
w = polynomial(speye(lw),w,h.var,[lw 1],chkval);


%----------------------------------------------------------------------
% Step 2) Build a matrix representation, L, of the linear operator 
%   which maps matrices to polynomials, Q --> w'*Q*w.  L and
%   Rbasis satisfy w'*Q*w=Rbasis'*L*Q(:)
%----------------------------------------------------------------------
[L,Rbasis] = buildLmat(w);  
lr = length(Rbasis);

%----------------------------------------------------------------------
% Step 3) Write p(x) and g(x) in terms of Rbasis:  
%   p(x) = Rbasis'*p_R and g(x) = Rbasis'*g_R
%----------------------------------------------------------------------
p_R = poly2basis(p,Rbasis);
if ~isempty(g)
    g_R = poly2basis(g,Rbasis);
else
    g_R = [];
end

%----------------------------------------------------------------------
% Step 4) p(x)+g(x)*c = w(x)'*Q*w(x) is equivalent to:
%   Rbasis'*(p_R + g_R*c) = Rbasis'*L*Q(:)
% This is equivalent to the matrix/vector relation
%   L*Q(:) = p_R - g_R*c 
% Rewrite this in the form:
%   Aq*Q(:) = b + Ac*c
%----------------------------------------------------------------------
b = p_R;
Aq = L;
Ac = g_R;

if nargin==1
    Ac = Rbasis;
end
