function [p,C,Z] = polydecvar(cstr,Z,type)
% function [p,C,z] = polydecvar(cstr,z,type)
%
% DESCRIPTION 
%   This function creates a polynomial decision variable in either vector
%   or Gram matrix form. If type='vec' then the polynomial decision 
%   variable is created in the form p = C'*z where C is a column vector of 
%   coefficients.  If type='mat' then the polynomial decision variable is 
%   created with the Gram matrix form: p=z'*C*z where C is a matrix of 
%   coefficients.  
%
%   If p will itself be constrained to be a SOS in the optimization then 
%   the Gram matrix form should be used.  If p will not be constrained
%   to be a SOS then the vector form should be used.  This enables the
%   SOS solver to efficiently solve the problem.
%
% INPUTS 
%   cstr: Character string to be used in creating the coefficient vector.
%   z: Nz-by-1 column vector of monomials
%   type:  String specifying the decision variable type.  type = 'vec'
%       (default) or type = 'mat'.
%
% OUTPUTS
%   p: polynomial decision variable
%   C: If type='vec' then C is an Nz-by-1 column vector of pvars with
%        names specified by cstr, e.g. if cstr = 'c' then C is [c_1_1;
%        ... c_2_1; c_Nz_1]. If type='mat' then C is an Nz-by-Nz matrix 
%        of pvars.
%   z: Nz-by-1 column vector of monomials. This is the same as the input
%        vector z.
%
% SYNTAX
%   [p,C,z] = polydecvar(cstr,z,type)
%

%  PJS 4/25/2009  Initial Coding

if nargin==2
    if isa(cstr,'polynomial') 
        type = 'getCZ';
    else
        type = 'vec';
    end
elseif nargin~=3
    error('Syntax: [p,C,Z] = polydecvar(cstr,Z,type)')
end
   
if strcmp(type,'vec');
    % Create poly dec var using coefficient vector form

    % Get Z degmat with same ordering as the monomials appear in Z 
    % XXX We are assuming that coef mat is a permutation of Identity
    Z = polynomial(Z(:));
    Zdeg = Z.coefficient'*Z.degmat;
    lz = length(Z);
            
    % Create coefficient vector
    C = mpvar(cstr,lz,1);
    Cvar = C.varname;
        
    % Create poly dec var
    pcoef = sparse(ones(lz,1));
    pdeg = [speye(lz) Zdeg];
    pvarname = [Cvar; Z.var];
    chkval = 0; % skip validity check
    p = polynomial(pcoef,pdeg,pvarname,[1 1],chkval);    
    
elseif strcmp(type,'mat');
    % Create poly dec var using Gram Matrix form

    % Get Z degmat with same ordering as the monomials appear in Z 
    % XXX We are assuming that coef mat is a permutation of Identity
    Z = polynomial(Z(:));
    Zdeg = Z.coefficient'*Z.degmat;
    lz = length(Z);
    
    % Create coefficient matrix
    C = mpvar(cstr,lz,lz);
    Cvar = C.varname;
    
    % Create poly dec var
    pcoef = sparse( ones(lz^2,1) );

    nxvar = size(Zdeg,2);
    pdeg = repmat(Zdeg,[lz 1]) + ...
        reshape(repmat(Zdeg(:)',[lz 1]),[lz^2 nxvar]);    
%     pdeg = sparse(lz^2,nxvar);
%     for i1=1:lz
%         idx = lz*(i1-1) + (1:lz);
%         pdeg(idx,:) = Zdeg+repmat(Zdeg(i1,:),[size(Zdeg,1) 1]);                
%     end
    pdeg = [speye(lz^2) pdeg];

    
    pvarname = [Cvar(:); Z.var];
    p = polynomial(pcoef,pdeg,pvarname,[1 1]);
    
elseif strcmp(type,'getCZ');
    % Undocumented syntax used in sosopt: [p,C,Z]=polydecvar(p,x)
    % This syntax attempts to find a monomials vector Z(x) and a coef
    % matrix C such that p=Z'*C*Z.  The function makes assumptions about
    % the form of Z and C (which will be satisfied if p was constructed
    % with the 'mat' option of polydecvar).  C,Z are returned as empty if 
    % the assumed decomposition is not successful.
     
    % Rename inputs
    p = cstr;    
    x = Z;    
    
    % Get polynomial and variable info
    pcoef = p.coefficient;
    if any( pcoef(:)~=1 )
        p=[];C=[];Z=[];
        return;
    end
    pdeg =  p.degmat;
    pvarname = p.varname;
    ntp = size(pdeg,1);
    if isa(x,'polynomial')
        xvar = x.varname;
    else
        xvar = x;
    end
    
    % Find the coefficient variables 
    cvar = setdiff(pvarname,xvar);    
    if isempty(cvar) 
        p=[];C=[];Z=[];
        return;
    else        
        %Ctmp = polynomial(1,1,cvar(1),[1 1]);
        Ctmp = polynomial(cvar(1));
        if (ntp==1) && (p==Ctmp)
            C=Ctmp;
            Z=polynomial(1);
            return;
        end
    end
    lz = sqrt( length(cvar) );
    
    % Create coefficient matrix, C
    idx = strfind(cvar{1},'_');
    if ~isempty(idx) && idx(1)>1 && floor(lz)==ceil(lz)
        cstr = cvar{1}(1:idx(1)-1);
        C = mpvar(cstr,lz,lz);
    else
        p=[];C=[];Z=[];
        return;
    end
    
    % Create monomials vector, Z(x)
    % (xvar here should be the same as x.var but we need the ordering of
    %  xvar to be the same as the ordering of xidx into the degree matrix)
    [xvar,xidx] = setdiff(pvarname,cvar);    
    if isempty(xvar)
        p=[];C=[];Z=[];
        return;
    end        
    xdeg = pdeg(:,xidx);
    nxvar = length(xvar);
    Zdeg = sparse(lz,nxvar);
    for i1=1:lz; 
        coefij = [cstr '_' int2str(i1) '_' int2str(i1)];    
        vidx = strmatch(coefij,pvarname,'exact');
        if isempty(vidx)
            p=[];C=[];Z=[];
            return;            
        end
        tidx = find(pdeg(:,vidx));
        Zdeg(i1,:) = xdeg(tidx,:)/2;
    end
    tmp = Zdeg(:);
    if any( floor(tmp)~=ceil(tmp))
        p=[];C=[];Z=[];
        return;
    end
    Z = polynomial(speye(lz),Zdeg,xvar,[lz 1]);

    % Check if t=Z'*C*Z is the same as the original poly p
    tcoef = sparse( ones(lz^2,1) );    
    tdeg = repmat(Zdeg,[lz 1]) + ...
        reshape(repmat(Zdeg(:)',[lz 1]),[lz^2 nxvar]);    
%     tdeg = sparse(lz^2,nxvar);
%     for i1=1:lz
%         idx = lz*(i1-1) + (1:lz);
%         tdeg(idx,:) = Zdeg+repmat(Zdeg(i1,:),[size(Zdeg,1) 1]);                
%     end
    tdeg = [C.degmat tdeg];    
    tvarname = [C.varname; xvar];
    t = polynomial(tcoef,tdeg,tvarname,[1 1]);
    if (p~=t)
        % Our assumed decomposition was not valid
        p=[];C=[];Z=[];
        return;
    end    
else
    error('Type must be either ''vec'' or ''mat''.')
end


