function [xs,ys,info] = sedumi2sdpam(As,bs,cs,K,opts); 
% function [x,y,info] = sedumi2sdpam(A,b,c,K,opts);
%
% DESCRIPTION 
%   This function solves a cone optimization specified in Sedumi format
%   using the SDPAM solver.  The function converts the inputs from
%   Sedumi format to SDPAM format, solves the problem with SDPAM, and
%   converts the SDPAM outputs back into Sedumi output format.
%
% INPUTS 
%   A,b,c,K,opts:  Cone optimization problem in Sedumi format.  See 
%       Sedumi documentation for more details.  SOCP constraints
%       are not handled by SDPAM.
%       
% OUTPUTS
%   x,y: Optimization results in Sedumi output format. See Sedumi
%       documentation.
%   info: SDPAM solver info.  See SDPAM documentation.
%
% SYNTAX 
%   [x,y,info] = sedumi2sdpam(A,b,c,K,opts);
%
% EXAMPLE
%
% 3/6/08  PJS     Initial Coding

if ~isfield(K,'f')
    K.f = [];
end
if length(K.s)==1 & K.s==0
    K.s = [];
end

% SDPAM does not handle free variables in the primal form. Split free 
% variabless into positive/negative parts.    
[A2,b2,c2,K2] = splitfreevars(As,bs,cs,K); 

% Convert from Sedumi Format to SDPAM Format
mDIM = size(A2,1);   % Number of Sedumi Dual Variables
cSDPAM = -full(b2(:))';
nBLOCK = ( K2.l>0 ) + length(K2.s);
F = cell(nBLOCK, mDIM+1);

blockidx = 1;
ptr = 0;
bLOCKsTRUCT = [];

n = K2.l;
if (n > 0)    
    bLOCKsTRUCT = [bLOCKsTRUCT -n];
    Ftmp = -[c2(1:n) A2(:,1:n)' ];
    F(1,:) = mat2cell( Ftmp , n ,ones(1,mDIM+1) );    
    blockidx = blockidx+1;
    ptr =n;
end
for i1 = 1:length(K2.s)
    n = K2.s(i1);
    bLOCKsTRUCT = [bLOCKsTRUCT n];
    idx = ptr+(1:n^2);
    Ftmp = reshape(-[c2(idx) A2(:,idx)'],n,n*(mDIM+1));
    F(blockidx,:) = mat2cell(Ftmp,n,repmat(n,1,mDIM+1));
    blockidx = blockidx+1;
    ptr = ptr+n^2;
end

%XXXX If opts.lambdaStar is too small then sdpam might
% only quit after one iteration.  Need to increase lambdaStar if
% this happens. Perhaps try several times?

% Call SDPAM
% if ~isfield(opts,'epsilonDash')
%     opts.epsilonDash = 1e-9;
% end
if ~isfield(opts,'print');
    opts.print = 'no';
end
[objVal,x,X,Y,info] = sdpam(mDIM,nBLOCK,bLOCKsTRUCT,cSDPAM,F,[],[],[],opts);
%[objVal,x,X,Y,info] = sdpam(mDIM,nBLOCK,bLOCKsTRUCT,cSDPAM,F);

% XXX For debug intialize sdpam at the optimal solution from sedumi
%[xs2,ys2,solverinfo] = sedumi(As,bs,cs,K,opts);
% x0 = ys2; X0 = reshape(cs-As'*ys2,K.s,K.s); Y0 = reshape(xs2,K.s,K.s);
% [objVal,x,X,Y,info] = sdpam(mDIM,nBLOCK,bLOCKsTRUCT,cSDPAM,F,x0,{X0},{Y0});

% XXX Try with yalmip 
% x = sdpvar(size(As,1),1);
% Fy = [];
% for i1=1:nBLOCK
%    Ftmp = -F{i1,1};
%    for i2=1:mDIM
%       Ftmp = Ftmp+F{i1,i2+1}*x(i2);
%    end
%    Fy = Fy+set(Ftmp);
% end
% opts = sdpsettings('solver','sdpa');
% d=solvesdp(Fy,cSDPAM*x,opts); xx=double(x);
% %%%%Fy = -F{1}; for i1=1:76; Fy = Fy+F{i1+1}*x(i1); end

% Convert solution from DSDP format to Sedumi format
ys = x;
Ndl = (K2.l>0);
if Ndl
    xs = Y{1}(:);
else
    xs = [];
end
for i1=1:length(K2.s);
    xs = [xs; Y{i1+Ndl}(:)];
end

% Recombine positive/negative parts of the free variables 
if ~isempty(K.f)
    xs = [xs(K.f+1 : 2*K.f) - xs(1:K.f); xs(2*K.f+1:end)];    
end

%keyboard