function  [info,dopt,sossol] = sosd(sosdata,opts)
% function [info,dopt,sossol] = sosd(sosdata,opts)
%
% DESCRIPTION 
%   This function solves the SOS optimization or feasibility problem in
%   kernel (dual) form.  An SOS optimization contains constraints of the
%   form (see sosopt for more details):
%               s(x,d) is sum of squares
%   where d are the decision variables and x are the polynomial variables.
%   Each SOS constraint is equivalent to:
%               s(x,d)=w(x)'*Q*w(x)
%               Q>=0
%   The constraint s(x,d)=w(x)'*Q*w(x) is simply a linear equality 
%   constraint between the entries of Q, the decision variables d, and 
%   the numeric coefficients of s.
%   
%   This function solves the optimization by constructing a particular
%   solution and a basis for the null space of this linear equality 
%   constraint.  Specifically, matrices Q0, D1,...,Dk are constructed
%   such that w(x)'*(Q0 - sum_i d(i)*Di)*w(x) is equal to s(x,d).  Also,
%   a set of basis matrices N1,...,Nj are constructed such that 
%   w(x)'*Ni*w(x) = 0.  Finally, the SOS constraint is replaced with
%   the linear matrix inequality constraint:
%            Q0 - sum_i di*Di -sum_i ni *Ni >=0.
%
% See also sosopt, sosp

% 2/29/08  PJS     Initial Coding
% 11/11/08 PJS     Simple mod to handle sos var representation
% 4/25/09 PJS      Clean up documentation and code for pure sos vars
% 5/6/09 PJS       Modify to allow set-up call

%---------------------------------------------------------------------
% Formulate SOS Optimization as an SDP
%---------------------------------------------------------------------
if isfield(sosdata,'sdpdata')
    % Input includes all SOS/SDP problem set-up information
    info = sosdata;
    sosdata = info.sosdata;
    sdpdata = info.sdpdata;
    sos2sdp = info.sos2sdp;
    opts = info.opts;

    % Get additional data needed to compute dopt and sossol
    sosconstr=sosdata.sosconstr;
    Nsc =length(sosconstr);
    decvars = sosdata.decvars;
    Ndv = length(decvars);
    scaling = sosdata.scaling;
    
    w = sos2sdp.w;
    
    Ast = (sdpdata.A)';
    cs = sdpdata.c;    
    K = sdpdata.K;
else
    % Get data about SOS problem formulation
    sosconstr=sosdata.sosconstr;
    x=sosdata.x;
    decvars = sosdata.decvars;
    obj = sosdata.obj;
    lpconstr = sosdata.lpconstr;
    scaling = sosdata.scaling;

    Nsc =length(sosconstr);
    Ndv = length(decvars);

    %---------------------------------------------------------------------
    % Define SOS/LP Cost and Constraints in Sedumi Dual Form
    %       max bs'*ys   s.t. cs-As'*ys  >= 0
    %
    % SOS Constraints: The constraint s(x,d) is SOS is equivalent to the 
    %     constraints s(x,d) = w(x)'*Q*w(x) and Q>=0.  These can be expressed
    %     as Q0 - sum_i di*Di -sum_i ni *Ni >=0.
    % LP Constraints: A*d<=b is defined as b-A*d>=0.
    %
    % Sedumi dual form orders the LP constraints before the LMI constraints.
    % The Sedumi decision variables are ordered as:
    %       xs=[d; n1; ....; nlast]
    % where d are the free decision variables, ni are the null space 
    % variables needed to express the SOS constraints.
    %---------------------------------------------------------------------

    % Define LP constraints: b-A*d>=0
    K.l = 0;
    Ast = sparse([]);    % Ast = As'
    cs = sparse([]);
    if ~isempty(lpconstr)
        % Define LP constraint dimension
        A = lpconstr.A;
        b = lpconstr.b;
        d = lpconstr.d; 
        Nconstr = length(b);
        K.l = Nconstr;

        % Find the free variables involved in the LP constraint
        [isdec,didx]=ismember(char(d),decvars);
        if any( isdec~=1 )
            error('LP constraints must only contain decision variables');
        end

        % Stack LP constraints below preceding constraints
        Ast(1:Nconstr,didx) = A;
        cs = [cs; b];            
    end

    % Define SOS Constraints: Q0 - sum_i di*Di -sum_i ni *Ni >=0
    K.s = [];
    wi = cell(Nsc,1);
    for i1=1:Nsc
        % i^th constraint is s(x,d) is SOS where d are the decision vars
        s = sosconstr{i1};
        if ~isa(s,'polynomial')
            % Pure SOS Var constraint -- Grab the polynomial form and w(x)
            w = s{2};
            s = s{1};
        else
            w = [];
        end


        % Collect s(x,d) into the form g0(x)+g1(x)*d1 + ... + gN(x)*dN
        % where di are decision variables.
        [g0,g,d] = collect(s,x);
        if ispvar(d)
            % Find the decision variables involved in the SOS constraint
            [isdec,didx]=ismember(char(d),decvars);            
        elseif d==0
            isdec=0; 
            didx=[];
        else
            error('SOS constraint must be linear in the decision variables');
        end

        % Express g0(x)+g(x)*d is SOS as 
        %     g0(x)+g(x)*d = w(x)'*Q*w(x),  Q>=0  (*)
        % Then find Q0, Ni and Di such that:
        %     w'*Q0*w = g0, w'*Ni*w=0, and w'*Di*w = gi
        % Thus (*) is equivalent to the LMI constraint:
        %     Q0 + sum_i ni*Ni + sum_i di*Di>=0    
        if any(isdec)
            [w,Q0,N,D] = gramsol(g0,g,w);  
        else
            [w,Q0,N] = gramsol(g0,[],w);          
        end
        wi{i1} = w;

        % Add this constraint to the SOS problem
        % Null space variables are added to the end of the Sedumi decision
        % variable vector.  The minus signs account for the sign in the
        % Sedumi dual constraint: c-A'*y>=0
        lw = length(w);
        K.s = [K.s lw];      
        if isempty(Ast)
            Ast = [ sparse(lw^2,Ndv) -N];
        else
            Ast = blkdiag(Ast,-N);
        end
        if any(isdec)
            Ast(end-lw^2+1:end,didx) = -D;        
        end    
        cs = [cs; Q0];    
    end
    As = Ast';
    w = wi;
    
    %---------------------------------------------------------------------
    % Create the objective function vector
    %---------------------------------------------------------------------
    if isempty(obj)
        % Feasibility Problem
        bs = sparse( size(Ast,2) , 1 );
    else
        % Objective function is obj = bs'*ys   
        [g0,g,d] = collect(obj,x);
        if (~ispvar(d) && d~=0) ||  (g0.nvar ~=0) || (g.nvar~=0)
            error('SOS constraint must be linear in the decision variables');
        end
        g0 = double(g0);
        [isdec,didx]=ismember(char(d),decvars);    
        bs = sparse( size(Ast,2) , 1 );

        % SOS programming is formulated as a minimization but the Sedumi 
        % dual form is a maximization.  Flip sign of bs to account for this.
        bs(didx) = -double(g);
    end  
    
    %---------------------------------------------------------------------
    % Create info data structure 
    %---------------------------------------------------------------------

    % sdpdata
    sdpdata.A = As;
    sdpdata.b = bs;
    sdpdata.c = cs;
    sdpdata.K = K;

    % sos2sdp: w, dv2x, dv2y, Nfv, Nlv, orderidx
    sos2sdp.w = w;
    sos2sdp.dv2x = [];
    sos2sdp.dv2y = 1:Ndv;
    sos2sdp.Nfv = [];
    sos2sdp.Nlv = [];
    sos2sdp.orderidx = [];

    % Pack all data into info
    info.sosdata = sosdata;
    info.sdpdata = sdpdata;
    info.sos2sdp = sos2sdp;
    info.opts = opts;
    
end
    
%---------------------------------------------------------------------
% Solve the problem 
%---------------------------------------------------------------------
if strcmpi(opts.solver,'setup')
    % Return with problem formulation for setup call
    sdpsol.x = [];
    sdpsol.y = [];
    sdpsol.solverinfo = [];
    info.sdpsol = sdpsol;

    dopt =[];
    sossol = [];
    return;
end
[xs,ys,solverinfo] = solvesossdp(sdpdata,opts); 

% sdpsol: x,y, solverinfo
sdpsol.x = xs;
sdpsol.y = ys;
sdpsol.solverinfo = solverinfo;
info.sdpsol = sdpsol;

%---------------------------------------------------------------------
% Create sossol and dopt for output
%---------------------------------------------------------------------

% Create dopt
if Ndv ~= 0
    dopt = [decvars(:) num2cell(ys(1:Ndv))];
else
    dopt = [];
end

if nargout==3
    % Create sossol 
    % The call to subs is only moderately fast so only do this if needed
    cnt = K.l;
    sossol = cell(Nsc,3);
    for i1 = 1:Nsc
        s = sosconstr{i1};
        if ~isa(s,'polynomial')
            % SOS variable constraint
            s = s{1};
        end
        if Ndv~=0
            s = subs(s,dopt); 
        end
        idx = cnt+(1:K.s(i1)^2);
        Qvec = cs(idx) - Ast(idx,:)*ys;
        Q = reshape( Qvec , K.s(i1), K.s(i1) ); 

        % Unscale solution
        scl = scaling(i1);
        if scl~=1
            s.coefficient = s.coefficient*scl;
            Q = Q*scl;
        end       
        
        % Store Gram Matrix solution for output
        sossol{i1,1} = s;
        sossol{i1,2} = w{i1};   
        sossol{i1,3} = Q;                
        
        cnt = idx(end);
    end
        
end